! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
! 
! Linear Algebra Data and Routines File
! 
! Generated by KPP-2.2.1_rs5 symbolic chemistry Kinetics PreProcessor
!       (http://www.cs.vt.edu/~asandu/Software/KPP)
! KPP is distributed under GPL, the general public licence
!       (http://www.gnu.org/copyleft/gpl.html)
! (C) 1995-1997, V. Damian & A. Sandu, CGRER, Univ. Iowa
! (C) 1997-2005, A. Sandu, Michigan Tech, Virginia Tech
!     With important contributions from:
!        M. Damian, Villanova University, USA
!        R. Sander, Max-Planck Institute for Chemistry, Mainz, Germany
! 
! File                 : gckpp_LinearAlgebra.f90
! Time                 : Mon Oct 19 13:12:24 2015
! Working directory    : /home/mpayer/KPP
! Equation file        : gckpp.kpp
! Output root filename : gckpp
! 
! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~



MODULE gckpp_LinearAlgebra

  USE gckpp_Parameters
  USE gckpp_JacobianSP

  IMPLICIT NONE

CONTAINS


! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
! 
! SPARSE_UTIL - SPARSE utility functions
!   Arguments :
! 
! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~


! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
SUBROUTINE KppDecomp( JVS, IER )
! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
!        Sparse LU factorization
! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

  USE gckpp_Parameters
  USE gckpp_JacobianSP

      INTEGER  :: IER
      REAL(kind=dp) :: JVS(LU_NONZERO), W(NVAR), a
      INTEGER  :: k, kk, j, jj

      a = 0. ! mz_rs_20050606
      IER = 0
      DO k=1,NVAR
        ! mz_rs_20050606: don't check if real value == 0
        ! IF ( JVS( LU_DIAG(k) ) .EQ. 0. ) THEN
        IF ( ABS(JVS(LU_DIAG(k))) < TINY(a) ) THEN
            IER = k
            RETURN
        END IF
        DO kk = LU_CROW(k), LU_CROW(k+1)-1
              W( LU_ICOL(kk) ) = JVS(kk)
        END DO
        DO kk = LU_CROW(k), LU_DIAG(k)-1
            j = LU_ICOL(kk)
            a = -W(j) / JVS( LU_DIAG(j) )
            W(j) = -a
            DO jj = LU_DIAG(j)+1, LU_CROW(j+1)-1
               W( LU_ICOL(jj) ) = W( LU_ICOL(jj) ) + a*JVS(jj)
            END DO
         END DO
         DO kk = LU_CROW(k), LU_CROW(k+1)-1
            JVS(kk) = W( LU_ICOL(kk) )
         END DO
      END DO
      
END SUBROUTINE KppDecomp


! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
SUBROUTINE KppDecompCmplx( JVS, IER )
! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
!        Sparse LU factorization, complex
! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

  USE gckpp_Parameters
  USE gckpp_JacobianSP

      INTEGER        :: IER
      DOUBLE COMPLEX :: JVS(LU_NONZERO), W(NVAR), a
      REAL(kind=dp)  :: b = 0.0
      INTEGER        :: k, kk, j, jj

      IER = 0
      DO k=1,NVAR
        IF ( ABS(JVS(LU_DIAG(k))) < TINY(b) ) THEN
            IER = k
            RETURN
        END IF
        DO kk = LU_CROW(k), LU_CROW(k+1)-1
              W( LU_ICOL(kk) ) = JVS(kk)
        END DO
        DO kk = LU_CROW(k), LU_DIAG(k)-1
            j = LU_ICOL(kk)
            a = -W(j) / JVS( LU_DIAG(j) )
            W(j) = -a
            DO jj = LU_DIAG(j)+1, LU_CROW(j+1)-1
               W( LU_ICOL(jj) ) = W( LU_ICOL(jj) ) + a*JVS(jj)
            END DO
         END DO
         DO kk = LU_CROW(k), LU_CROW(k+1)-1
            JVS(kk) = W( LU_ICOL(kk) )
         END DO
      END DO
      
END SUBROUTINE KppDecompCmplx


! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
SUBROUTINE KppDecompCmplxR( JVSR, JVSI, IER )
! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
!    Sparse LU factorization, complex
!   (Real and Imaginary parts are used instead of complex data type)     
! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

  USE gckpp_Parameters
  USE gckpp_JacobianSP

      INTEGER       :: IER
      REAL(kind=dp) :: JVSR(LU_NONZERO), JVSI(LU_NONZERO) 
      REAL(kind=dp) :: WR(NVAR), WI(NVAR), ar, ai, den
      INTEGER       :: k, kk, j, jj

      IER = 0
      ar  = 0.0
      DO k=1,NVAR
        IF (  ( ABS(JVSR(LU_DIAG(k))) < TINY(ar) ) .AND. &
              ( ABS(JVSI(LU_DIAG(k))) < TINY(ar) ) )  THEN
            IER = k
            RETURN
        END IF
        DO kk = LU_CROW(k), LU_CROW(k+1)-1
              WR( LU_ICOL(kk) ) = JVSR(kk)
              WI( LU_ICOL(kk) ) = JVSI(kk)
        END DO
        DO kk = LU_CROW(k), LU_DIAG(k)-1
            j = LU_ICOL(kk)
            den = JVSR(LU_DIAG(j))**2 + JVSI(LU_DIAG(j))**2
            ar = -(WR(j)*JVSR(LU_DIAG(j)) + WI(j)*JVSI(LU_DIAG(j)))/den
            ai = -(WI(j)*JVSR(LU_DIAG(j)) - WR(j)*JVSI(LU_DIAG(j)))/den
            WR(j) = -ar
            WI(j) = -ai
            DO jj = LU_DIAG(j)+1, LU_CROW(j+1)-1
               WR( LU_ICOL(jj) ) = WR( LU_ICOL(jj) ) + ar*JVSR(jj) - ai*JVSI(jj)
               WI( LU_ICOL(jj) ) = WI( LU_ICOL(jj) ) + ar*JVSI(jj) + ai*JVSR(jj)
            END DO
         END DO
         DO kk = LU_CROW(k), LU_CROW(k+1)-1
            JVSR(kk) = WR( LU_ICOL(kk) )
            JVSI(kk) = WI( LU_ICOL(kk) )
         END DO
      END DO

END SUBROUTINE KppDecompCmplxR


! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
SUBROUTINE KppSolveIndirect( JVS, X )
! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
!        Sparse solve subroutine using indirect addressing
! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

  USE gckpp_Parameters
  USE gckpp_JacobianSP

      INTEGER  :: i, j
      REAL(kind=dp) :: JVS(LU_NONZERO), X(NVAR), sum

      DO i=1,NVAR
         DO j = LU_CROW(i), LU_DIAG(i)-1 
             X(i) = X(i) - JVS(j)*X(LU_ICOL(j));
         END DO  
      END DO

      DO i=NVAR,1,-1
        sum = X(i);
        DO j = LU_DIAG(i)+1, LU_CROW(i+1)-1
          sum = sum - JVS(j)*X(LU_ICOL(j));
        END DO
        X(i) = sum/JVS(LU_DIAG(i));
      END DO
      
END SUBROUTINE KppSolveIndirect


! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
SUBROUTINE KppSolveTRIndirect( JVS, X )
! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
!        Complex sparse solve transpose subroutine using indirect addressing
! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

  USE gckpp_Parameters
  USE gckpp_JacobianSP

      INTEGER       :: i, j
      REAL(kind=dp) :: JVS(LU_NONZERO), X(NVAR)

      DO i=1,NVAR
        X(i) = X(i)/JVS(LU_DIAG(i))
        ! subtract all nonzero elements in row i of JVS from X
        DO j=LU_DIAG(i)+1,LU_CROW(i+1)-1
          X(LU_ICOL(j)) = X(LU_ICOL(j))-JVS(j)*X(i)
        END DO
      END DO

      DO i=NVAR, 1, -1
        ! subtract all nonzero elements in row i of JVS from X
        DO j=LU_CROW(i),LU_DIAG(i)-1
          X(LU_ICOL(j)) = X(LU_ICOL(j))-JVS(j)*X(i)
        END DO
      END DO
      
END SUBROUTINE KppSolveTRIndirect


! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
SUBROUTINE KppSolveCmplx( JVS, X )
! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
!        Complex sparse solve subroutine using indirect addressing
! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

  USE gckpp_Parameters
  USE gckpp_JacobianSP

      INTEGER        :: i, j
      DOUBLE COMPLEX :: JVS(LU_NONZERO), X(NVAR), sum

      DO i=1,NVAR
         DO j = LU_CROW(i), LU_DIAG(i)-1 
             X(i) = X(i) - JVS(j)*X(LU_ICOL(j));
         END DO  
      END DO

      DO i=NVAR,1,-1
        sum = X(i);
        DO j = LU_DIAG(i)+1, LU_CROW(i+1)-1
          sum = sum - JVS(j)*X(LU_ICOL(j));
        END DO
        X(i) = sum/JVS(LU_DIAG(i));
      END DO
      
END SUBROUTINE KppSolveCmplx

! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
SUBROUTINE KppSolveCmplxR( JVSR, JVSI, XR, XI )
! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
!   Complex sparse solve subroutine using indirect addressing
!   (Real and Imaginary parts are used instead of complex data type)     
! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

  USE gckpp_Parameters
  USE gckpp_JacobianSP

      INTEGER       ::  i, j
      REAL(kind=dp) ::  JVSR(LU_NONZERO), JVSI(LU_NONZERO), XR(NVAR), XI(NVAR), sumr, sumi, den

      DO i=1,NVAR
         DO j = LU_CROW(i), LU_DIAG(i)-1 
             XR(i) = XR(i) - (JVSR(j)*XR(LU_ICOL(j)) - JVSI(j)*XI(LU_ICOL(j)))
             XI(i) = XI(i) - (JVSR(j)*XI(LU_ICOL(j)) + JVSI(j)*XR(LU_ICOL(j)))
         END DO  
      END DO

      DO i=NVAR,1,-1
        sumr = XR(i); sumi = XI(i)
        DO j = LU_DIAG(i)+1, LU_CROW(i+1)-1
            sumr = sumr - (JVSR(j)*XR(LU_ICOL(j)) - JVSI(j)*XI(LU_ICOL(j)))
            sumi = sumi - (JVSR(j)*XI(LU_ICOL(j)) + JVSI(j)*XR(LU_ICOL(j)))
        END DO
        den   = JVSR(LU_DIAG(i))**2 + JVSI(LU_DIAG(i))**2
        XR(i) = (sumr*JVSR(LU_DIAG(i)) + sumi*JVSI(LU_DIAG(i)))/den
        XI(i) = (sumi*JVSR(LU_DIAG(i)) - sumr*JVSI(LU_DIAG(i)))/den
      END DO
      
END SUBROUTINE KppSolveCmplxR


! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
SUBROUTINE KppSolveTRCmplx( JVS, X )
! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
!        Complex sparse solve transpose subroutine using indirect addressing
! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

  USE gckpp_Parameters
  USE gckpp_JacobianSP

      INTEGER        :: i, j
      DOUBLE COMPLEX :: JVS(LU_NONZERO), X(NVAR)

      DO i=1,NVAR
        X(i) = X(i)/JVS(LU_DIAG(i))
        ! subtract all nonzero elements in row i of JVS from X
        DO j=LU_DIAG(i)+1,LU_CROW(i+1)-1
          X(LU_ICOL(j)) = X(LU_ICOL(j))-JVS(j)*X(i)
        END DO
      END DO

      DO i=NVAR, 1, -1
        ! subtract all nonzero elements in row i of JVS from X
        DO j=LU_CROW(i),LU_DIAG(i)-1
          X(LU_ICOL(j)) = X(LU_ICOL(j))-JVS(j)*X(i)
        END DO
      END DO
      
END SUBROUTINE KppSolveTRCmplx


! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
SUBROUTINE KppSolveTRCmplxR( JVSR, JVSI, XR, XI )
! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
!   Complex sparse solve transpose subroutine using indirect addressing
!   (Real and Imaginary parts are used instead of complex data type)     
! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

  USE gckpp_Parameters
  USE gckpp_JacobianSP

      INTEGER       ::  i, j
      REAL(kind=dp) ::  JVSR(LU_NONZERO), JVSI(LU_NONZERO), XR(NVAR), XI(NVAR), den

      DO i=1,NVAR
        den   = JVSR(LU_DIAG(i))**2 + JVSI(LU_DIAG(i))**2
        XR(i) = (XR(i)*JVSR(LU_DIAG(i)) + XI(i)*JVSI(LU_DIAG(i)))/den
        XI(i) = (XI(i)*JVSR(LU_DIAG(i)) - XR(i)*JVSI(LU_DIAG(i)))/den
        ! subtract all nonzero elements in row i of JVS from X
        DO j=LU_DIAG(i)+1,LU_CROW(i+1)-1
          XR(LU_ICOL(j)) = XR(LU_ICOL(j))-(JVSR(j)*XR(i) - JVSI(j)*XI(i))
          XI(LU_ICOL(j)) = XI(LU_ICOL(j))-(JVSI(j)*XR(i) + JVSR(j)*XI(i))
        END DO
      END DO

      DO i=NVAR, 1, -1
        ! subtract all nonzero elements in row i of JVS from X
        DO j=LU_CROW(i),LU_DIAG(i)-1
          XR(LU_ICOL(j)) = XR(LU_ICOL(j))-(JVSR(j)*XR(i) - JVSI(j)*XI(i))
          XI(LU_ICOL(j)) = XI(LU_ICOL(j))-(JVSI(j)*XR(i) + JVSR(j)*XI(i))
        END DO
      END DO
      
END SUBROUTINE KppSolveTRCmplxR


!
! Next few commented subroutines perform sparse big linear algebra
!
!! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
!SUBROUTINE KppDecompBig( JVS, IP, IER )
!! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
!!        Sparse LU factorization
!!        for the Runge Kutta (3n)x(3n) linear system
!! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
!
!  USE gckpp_Parameters
!  USE gckpp_JacobianSP
!
!      INTEGER  :: IP3(3), IER, IP(3,NVAR)
!      REAL(kind=dp) :: JVS(3,3,LU_NONZERO), W(3,3,NVAR), a(3,3), E(3,3)
!      INTEGER  :: k, kk, j, jj
!
!      a = 0.0d0
!      IER = 0
!      DO k=1,NVAR
!        DO kk = LU_CROW(k), LU_CROW(k+1)-1
!              W( 1:3,1:3,LU_ICOL(kk) ) = JVS(1:3,1:3,kk)
!        END DO
!        DO kk = LU_CROW(k), LU_DIAG(k)-1
!            j = LU_ICOL(kk)
!            E(1:3,1:3) = JVS( 1:3,1:3,LU_DIAG(j) )
!            ! CALL DGETRF(3,3,E,3,IP3,IER) 
!            CALL FAC3(E,IP3,IER)
!            IF ( IER /= 0 )  RETURN
!            ! a = W(j) / JVS( LU_DIAG(j) )
!            a(1:3,1:3) = W( 1:3,1:3,j )
!            ! CALL DGETRS ('N',3,3,E,3,IP3,a,3,IER) 
!            CALL SOL3('N',E,IP3,a(1,1))
!            CALL SOL3('N',E,IP3,a(1,2))
!            CALL SOL3('N',E,IP3,a(1,3))
!            W(1:3,1:3,j) = a(1:3,1:3)
!            DO jj = LU_DIAG(j)+1, LU_CROW(j+1)-1
!               W( 1:3,1:3,LU_ICOL(jj) ) = W( 1:3,1:3,LU_ICOL(jj) ) &
!                        - MATMUL( a(1:3,1:3) , JVS(1:3,1:3,jj) )
!            END DO
!         END DO
!         DO kk = LU_CROW(k), LU_CROW(k+1)-1
!            JVS(1:3,1:3,kk) = W( 1:3,1:3,LU_ICOL(kk) )
!         END DO
!      END DO
!
!      DO k=1,NVAR
!         ! CALL WGEFA(JVS(1,1,LU_DIAG(k)),3,3,IP(1,k),IER)
!         ! CALL DGETRF(3,3,JVS(1,1,LU_DIAG(k)),3,IP(1,k),IER)
!         CALL FAC3(JVS(1,1,LU_DIAG(k)),IP(1,k),IER)
!         IF ( IER /= 0 )  RETURN
!      END DO 
!      
!END SUBROUTINE KppDecompBig
!
!
!! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
!SUBROUTINE KppSolveBig( JVS, IP, X )
!! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
!!        Sparse solve subroutine using indirect addressing
!!        for the Runge Kutta (3n)x(3n) linear system
!! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
!
!  USE gckpp_Parameters
!  USE gckpp_JacobianSP
!
!      INTEGER  :: i, j, k, m, IP3(3), IP(3,NVAR), IER
!      REAL(kind=dp) :: JVS(3,3,LU_NONZERO), X(3,NVAR), sum(3)
!
!      DO i=1,NVAR
!        DO j = LU_CROW(i), LU_DIAG(i)-1 
!          !X(1:3,i) = X(1:3,i) - MATMUL(JVS(1:3,1:3,j),X(1:3,LU_ICOL(j)));
!          DO k=1,3
!            DO m=1,3
!              X(k,i) = X(k,i) - JVS(k,m,j)*X(m,LU_ICOL(j))
!            END DO
!          END DO
!        END DO  
!      END DO
!
!      DO i=NVAR,1,-1
!        sum(1:3) = X(1:3,i);
!        DO j = LU_DIAG(i)+1, LU_CROW(i+1)-1
!          !sum(1:3) = sum(1:3) - MATMUL(JVS(1:3,1:3,j),X(1:3,LU_ICOL(j)));
!          DO k=1,3
!            DO m=1,3
!              sum(k) = sum(k) - JVS(k,m,j)*X(m,LU_ICOL(j))
!            END DO
!          END DO
!        END DO
!        ! X(i) = sum/JVS(LU_DIAG(i));
!        ! CALL DGETRS ('N',3,1,JVS(1:3,1:3,LU_DIAG(i)),3,IP(1,i),sum,3,0) 
!        ! CALL WGESL('N',JVS(1,1,LU_DIAG(i)),3,3,IP(1,i),sum)
!        CALL SOL3('N',JVS(1,1,LU_DIAG(i)),IP(1,i),sum)
!        X(1:3,i) = sum(1:3)
!      END DO
!      
!END SUBROUTINE KppSolveBig
!
!
!! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
!SUBROUTINE KppSolveBigTR( JVS, IP, X )
!! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
!!        Big sparse transpose solve using indirect addressing
!! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
!
!  USE gckpp_Parameters
!  USE gckpp_JacobianSP
!
!      INTEGER       :: i, j, k, m, IP(3,NVAR)
!      REAL(kind=dp) :: JVS(3,3,LU_NONZERO), X(3,NVAR)
!
!      DO i=1,NVAR
!        ! X(i) = X(i)/JVS(LU_DIAG(i))
!        CALL SOL3('T',JVS(1,1,LU_DIAG(i)),IP(1,i),X(1,i))
!        DO j=LU_DIAG(i)+1,LU_CROW(i+1)-1
!         !X(1:3,LU_ICOL(j)) = X(1:3,LU_ICOL(j)) &
!          !    - MATMUL( TRANSPOSE(JVS(1:3,1:3,j)), X(1:3,i) )
!          DO k=1,3
!            DO m=1,3
!              X(k,LU_ICOL(j)) = X(k,LU_ICOL(j)) - JVS(m,k,j)*X(m,i)
!            END DO
!          END DO
!       END DO
!      END DO
!
!      DO i=NVAR, 1, -1
!        DO j=LU_CROW(i),LU_DIAG(i)-1
!         !X(1:3,LU_ICOL(j)) = X(1:3,LU_ICOL(j)) &
!          !   - MATMUL( TRANSPOSE(JVS(1:3,1:3,j)), X(1:3,i) )
!          DO k=1,3
!            DO m=1,3
!              X(k,LU_ICOL(j)) = X(k,LU_ICOL(j)) - JVS(m,k,j)*X(m,i)
!            END DO
!          END DO
!       END DO
!      END DO
!      
!END SUBROUTINE KppSolveBigTR
!
!
!
!! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
!SUBROUTINE FAC3(A,IPVT,INFO)
!! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
!!     FAC3 FACTORS THE MATRIX A (3,3) BY
!!           GAUSS ELIMINATION WITH PARTIAL PIVOTING
!!     LINPACK - LIKE 
!!
!!     Remove comments to perform pivoting
!! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
!!
!      REAL(kind=dp) :: A(3,3)
!      INTEGER       :: IPVT(3),INFO
!!      INTEGER       :: L
!!      REAL(kind=dp) :: t, dmax, da, TMP(3)
!      REAL(kind=dp), PARAMETER :: ZERO = 0.0, ONE = 1.0
!
!      info = 0
!!      t = TINY(da)
!!      
!!      da = ABS(A(1,1)); L = 1
!!      IF ( ABS(A(2,1))>da ) THEN
!!        da = ABS(A(2,1)); L = 2
!!        IF ( ABS(A(3,1))>da ) THEN
!!          L = 3
!!        END IF  
!!      END IF  
!!      IPVT(1)  = L
!!      IF (L /=1 ) THEN
!!         TMP(1:3) = A(L,1:3)
!!         A(L,1:3) = A(1,1:3)
!!         A(1,1:3) = TMP(1:3)
!!      END IF
!!      IF (ABS(A(1,1)) < t) THEN
!!         info = 1
!!         return
!!      END IF   
!!
!      A(2,1) = A(2,1)/A(1,1)
!      A(2,2) = A(2,2) - A(2,1)*A(1,2)
!      A(2,3) = A(2,3) - A(2,1)*A(1,3)
!      A(3,1) = A(3,1)/A(1,1)
!      A(3,2) = A(3,2) - A(3,1)*A(1,2)
!      A(3,3) = A(3,3) - A(3,1)*A(1,3)
!      
!!      IPVT(2)  = 2
!!      IF (ABS(A(3,2))>ABS(A(2,2))) THEN
!!         IPVT(2)  = 3
!!         TMP(2:3) = A(3,2:3)
!!         A(3,2:3) = A(2,2:3)
!!         A(2,2:3) = TMP(2:3)
!!      END IF
!!      IF (ABS(A(2,2)) < t) THEN
!!         info = 1
!!         return
!!      END IF   
!!      
!      A(3,2)   = A(3,2)/A(2,2)
!      A(3,3)   = A(3,3) - A(3,2)*A(2,3)
!      IPVT(3)  = 3
!      
!END SUBROUTINE FAC3
!
!
!! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
!SUBROUTINE SOL3(Trans,A,IPVT,b)
!! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
!!     SOL3 solves the system 3x3
!!     A * x = b  or  trans(a) * x = b
!!     using the factors computed by WGEFA.
!!
!!     Trans      = 'N'   to solve  A*x = b ,
!!                = 'T'   to solve  transpose(A)*x = b
!!     LINPACK - LIKE 
!!
!!     Remove comments to use pivoting
!! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
!
!      CHARACTER     :: Trans
!      REAL(kind=dp) :: a(3,3),b(3)
!      INTEGER       :: IPVT(3)
!!      INTEGER       :: L
!!      REAL(kind=dp) :: TMP
!      
!      SELECT CASE (Trans)
!
!      CASE ('n','N')  !  Solve  A * x = b
!
!!     Solve  L*y = b
!!         L = IPVT(1)
!!         IF (L /= 1) THEN
!!            TMP = B(1); B(1) = B(L); B(L) = TMP
!!         END IF
!         b(2) = b(2)-A(2,1)*b(1)
!         b(3) = b(3)-A(3,1)*b(1)
!         
!!         L = IPVT(2)
!!         IF (L /= 2) THEN
!!            TMP = B(2); B(2) = B(L); B(L) = TMP
!!         END IF
!         b(3) = b(3)-A(3,2)*b(2)
!
!!     Solve  U*x = y
!         b(3) = b(3)/A(3,3)
!         b(2) = (b(2)-A(2,3)*b(3))/A(2,2)
!         b(1) = (b(1)-A(1,3)*b(3)-A(1,2)*b(2))/A(1,1)
!      
!      
!      CASE ('t','T')  !  Solve transpose(A) * x = b
!
!!      Solve transpose(U)*y = b
!         b(1) = b(1)/A(1,1)
!         b(2) = (b(2)-A(1,2)*b(1))/A(2,2)
!         b(3) = (b(3)-A(1,3)*b(1)-A(2,3)*b(2))/A(3,3)
!
!!      Solve transpose(L)*x = y
!         b(2) = b(2)-A(3,2)*b(3)
!!         L = ipvt(2)
!!         IF (L /= 2) THEN
!!            TMP = B(2); B(2) = B(L); B(L) = TMP
!!         END IF
!         b(1) = b(1)-A(3,1)*b(3)-A(2,1)*b(2)
!!         L = ipvt(1)
!!         IF (L /= 1) THEN
!!            TMP = B(1); B(1) = B(L); B(L) = TMP
!!         END IF
!   
!      END SELECT
!
!END SUBROUTINE SOL3

! End of SPARSE_UTIL function
! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~


! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
! 
! KppSolve - sparse back substitution
!   Arguments :
!      JVS       - sparse Jacobian of variables
!      X         - Vector for variables
! 
! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

SUBROUTINE KppSolve ( JVS, X )

! JVS - sparse Jacobian of variables
  REAL(kind=dp) :: JVS(LU_NONZERO)
! X - Vector for variables
  REAL(kind=dp) :: X(NVAR)

  X(57) = X(57)-JVS(194)*X(56)
  X(58) = X(58)-JVS(202)*X(53)
  X(73) = X(73)-JVS(281)*X(72)
  X(78) = X(78)-JVS(311)*X(41)-JVS(312)*X(70)
  X(79) = X(79)-JVS(318)*X(42)
  X(80) = X(80)-JVS(325)*X(74)
  X(86) = X(86)-JVS(373)*X(77)
  X(88) = X(88)-JVS(389)*X(50)
  X(92) = X(92)-JVS(419)*X(91)
  X(93) = X(93)-JVS(425)*X(62)
  X(95) = X(95)-JVS(442)*X(61)
  X(98) = X(98)-JVS(467)*X(43)-JVS(468)*X(45)-JVS(469)*X(51)-JVS(470)*X(62)-JVS(471)*X(64)-JVS(472)*X(80)-JVS(473)*X(82)&
            &-JVS(474)*X(93)
  X(99) = X(99)-JVS(497)*X(49)-JVS(498)*X(53)
  X(100) = X(100)-JVS(508)*X(78)-JVS(509)*X(82)-JVS(510)*X(89)-JVS(511)*X(97)
  X(101) = X(101)-JVS(524)*X(59)
  X(103) = X(103)-JVS(539)*X(77)
  X(104) = X(104)-JVS(549)*X(91)
  X(105) = X(105)-JVS(558)*X(65)-JVS(559)*X(92)
  X(108) = X(108)-JVS(583)*X(50)-JVS(584)*X(53)
  X(109) = X(109)-JVS(594)*X(61)-JVS(595)*X(92)
  X(111) = X(111)-JVS(613)*X(69)-JVS(614)*X(91)
  X(112) = X(112)-JVS(624)*X(72)
  X(113) = X(113)-JVS(632)*X(54)-JVS(633)*X(64)
  X(114) = X(114)-JVS(647)*X(69)-JVS(648)*X(72)-JVS(649)*X(78)-JVS(650)*X(82)-JVS(651)*X(86)-JVS(652)*X(87)-JVS(653)&
             &*X(102)-JVS(654)*X(111)-JVS(655)*X(112)
  X(115) = X(115)-JVS(673)*X(39)-JVS(674)*X(60)-JVS(675)*X(106)
  X(116) = X(116)-JVS(688)*X(68)-JVS(689)*X(88)-JVS(690)*X(108)-JVS(691)*X(110)
  X(117) = X(117)-JVS(703)*X(76)-JVS(704)*X(107)
  X(119) = X(119)-JVS(724)*X(59)-JVS(725)*X(83)-JVS(726)*X(106)-JVS(727)*X(118)
  X(120) = X(120)-JVS(739)*X(51)-JVS(740)*X(84)-JVS(741)*X(85)-JVS(742)*X(87)-JVS(743)*X(90)-JVS(744)*X(91)-JVS(745)&
             &*X(92)-JVS(746)*X(94)-JVS(747)*X(96)-JVS(748)*X(104)
  X(121) = X(121)-JVS(759)*X(68)-JVS(760)*X(69)-JVS(761)*X(72)-JVS(762)*X(78)-JVS(763)*X(79)-JVS(764)*X(82)-JVS(765)&
             &*X(86)-JVS(766)*X(94)-JVS(767)*X(95)-JVS(768)*X(96)-JVS(769)*X(102)-JVS(770)*X(103)-JVS(771)*X(107)-JVS(772)&
             &*X(109)-JVS(773)*X(110)-JVS(774)*X(111)-JVS(775)*X(112)-JVS(776)*X(116)-JVS(777)*X(120)
  X(122) = X(122)-JVS(801)*X(70)-JVS(802)*X(91)
  X(123) = X(123)-JVS(811)*X(75)-JVS(812)*X(89)-JVS(813)*X(91)-JVS(814)*X(97)-JVS(815)*X(118)
  X(124) = X(124)-JVS(824)*X(70)-JVS(825)*X(77)-JVS(826)*X(102)-JVS(827)*X(122)-JVS(828)*X(123)
  X(125) = X(125)-JVS(837)*X(48)-JVS(838)*X(51)-JVS(839)*X(52)-JVS(840)*X(59)-JVS(841)*X(63)-JVS(842)*X(65)-JVS(843)&
             &*X(68)-JVS(844)*X(69)-JVS(845)*X(70)-JVS(846)*X(71)-JVS(847)*X(77)-JVS(848)*X(78)-JVS(849)*X(79)-JVS(850)&
             &*X(83)-JVS(851)*X(86)-JVS(852)*X(89)-JVS(853)*X(90)-JVS(854)*X(91)-JVS(855)*X(92)-JVS(856)*X(95)-JVS(857)&
             &*X(97)-JVS(858)*X(99)-JVS(859)*X(100)-JVS(860)*X(101)-JVS(861)*X(102)-JVS(862)*X(103)-JVS(863)*X(104)-JVS(864)&
             &*X(105)-JVS(865)*X(106)-JVS(866)*X(107)-JVS(867)*X(108)-JVS(868)*X(109)-JVS(869)*X(110)-JVS(870)*X(111)&
             &-JVS(871)*X(112)-JVS(872)*X(113)-JVS(873)*X(114)-JVS(874)*X(115)-JVS(875)*X(116)-JVS(876)*X(117)-JVS(877)&
             &*X(118)-JVS(878)*X(119)-JVS(879)*X(120)-JVS(880)*X(122)-JVS(881)*X(123)-JVS(882)*X(124)
  X(126) = X(126)-JVS(905)*X(71)-JVS(906)*X(104)-JVS(907)*X(120)-JVS(908)*X(124)
  X(127) = X(127)-JVS(921)*X(76)-JVS(922)*X(77)-JVS(923)*X(107)-JVS(924)*X(122)
  X(128) = X(128)-JVS(935)*X(76)-JVS(936)*X(102)-JVS(937)*X(107)-JVS(938)*X(122)-JVS(939)*X(123)-JVS(940)*X(127)
  X(129) = X(129)-JVS(950)*X(65)-JVS(951)*X(71)-JVS(952)*X(72)-JVS(953)*X(73)-JVS(954)*X(78)-JVS(955)*X(79)-JVS(956)&
             &*X(82)-JVS(957)*X(83)-JVS(958)*X(87)-JVS(959)*X(101)-JVS(960)*X(102)-JVS(961)*X(103)-JVS(962)*X(104)-JVS(963)&
             &*X(105)-JVS(964)*X(106)-JVS(965)*X(112)-JVS(966)*X(116)-JVS(967)*X(118)-JVS(968)*X(119)-JVS(969)*X(120)&
             &-JVS(970)*X(122)-JVS(971)*X(123)-JVS(972)*X(124)-JVS(973)*X(126)-JVS(974)*X(127)-JVS(975)*X(128)
  X(130) = X(130)-JVS(992)*X(45)-JVS(993)*X(67)-JVS(994)*X(80)-JVS(995)*X(82)-JVS(996)*X(112)-JVS(997)*X(117)-JVS(998)&
             &*X(122)-JVS(999)*X(124)-JVS(1000)*X(127)-JVS(1001)*X(128)
  X(131) = X(131)-JVS(1012)*X(54)-JVS(1013)*X(60)-JVS(1014)*X(65)-JVS(1015)*X(85)-JVS(1016)*X(92)-JVS(1017)*X(105)&
             &-JVS(1018)*X(106)-JVS(1019)*X(109)-JVS(1020)*X(110)-JVS(1021)*X(113)-JVS(1022)*X(115)-JVS(1023)*X(117)&
             &-JVS(1024)*X(118)-JVS(1025)*X(120)-JVS(1026)*X(123)-JVS(1027)*X(124)-JVS(1028)*X(127)-JVS(1029)*X(128)&
             &-JVS(1030)*X(130)
  X(132) = X(132)-JVS(1047)*X(75)-JVS(1048)*X(89)-JVS(1049)*X(97)-JVS(1050)*X(106)-JVS(1051)*X(107)-JVS(1052)*X(110)&
             &-JVS(1053)*X(116)-JVS(1054)*X(117)-JVS(1055)*X(118)-JVS(1056)*X(123)-JVS(1057)*X(127)-JVS(1058)*X(128)&
             &-JVS(1059)*X(130)
  X(133) = X(133)-JVS(1072)*X(49)-JVS(1073)*X(61)-JVS(1074)*X(67)-JVS(1075)*X(75)-JVS(1076)*X(76)-JVS(1077)*X(80)&
             &-JVS(1078)*X(81)-JVS(1079)*X(82)-JVS(1080)*X(89)-JVS(1081)*X(97)-JVS(1082)*X(99)-JVS(1083)*X(101)-JVS(1084)&
             &*X(105)-JVS(1085)*X(107)-JVS(1086)*X(108)-JVS(1087)*X(109)-JVS(1088)*X(110)-JVS(1089)*X(111)-JVS(1090)*X(112)&
             &-JVS(1091)*X(117)-JVS(1092)*X(118)-JVS(1093)*X(119)-JVS(1094)*X(122)-JVS(1095)*X(123)-JVS(1096)*X(124)&
             &-JVS(1097)*X(126)-JVS(1098)*X(127)-JVS(1099)*X(128)-JVS(1100)*X(130)-JVS(1101)*X(132)
  X(134) = X(134)-JVS(1116)*X(36)-JVS(1117)*X(37)-JVS(1118)*X(38)-JVS(1119)*X(40)-JVS(1120)*X(53)-JVS(1121)*X(56)&
             &-JVS(1122)*X(57)-JVS(1123)*X(58)-JVS(1124)*X(62)-JVS(1125)*X(64)-JVS(1126)*X(88)-JVS(1127)*X(93)-JVS(1128)&
             &*X(108)-JVS(1129)*X(125)-JVS(1130)*X(126)-JVS(1131)*X(127)-JVS(1132)*X(128)-JVS(1133)*X(129)-JVS(1134)*X(130)&
             &-JVS(1135)*X(131)-JVS(1136)*X(132)-JVS(1137)*X(133)
  X(135) = X(135)-JVS(1151)*X(69)-JVS(1152)*X(73)-JVS(1153)*X(74)-JVS(1154)*X(75)-JVS(1155)*X(89)-JVS(1156)*X(97)&
             &-JVS(1157)*X(101)-JVS(1158)*X(110)-JVS(1159)*X(111)-JVS(1160)*X(112)-JVS(1161)*X(116)-JVS(1162)*X(118)&
             &-JVS(1163)*X(119)-JVS(1164)*X(122)-JVS(1165)*X(123)-JVS(1166)*X(124)-JVS(1167)*X(126)-JVS(1168)*X(128)&
             &-JVS(1169)*X(130)-JVS(1170)*X(132)-JVS(1171)*X(134)
  X(136) = X(136)-JVS(1184)*X(53)-JVS(1185)*X(64)-JVS(1186)*X(71)-JVS(1187)*X(72)-JVS(1188)*X(73)-JVS(1189)*X(78)&
             &-JVS(1190)*X(83)-JVS(1191)*X(84)-JVS(1192)*X(85)-JVS(1193)*X(87)-JVS(1194)*X(88)-JVS(1195)*X(91)-JVS(1196)&
             &*X(92)-JVS(1197)*X(94)-JVS(1198)*X(95)-JVS(1199)*X(96)-JVS(1200)*X(100)-JVS(1201)*X(102)-JVS(1202)*X(103)&
             &-JVS(1203)*X(104)-JVS(1204)*X(106)-JVS(1205)*X(107)-JVS(1206)*X(108)-JVS(1207)*X(109)-JVS(1208)*X(112)&
             &-JVS(1209)*X(114)-JVS(1210)*X(118)-JVS(1211)*X(119)-JVS(1212)*X(120)-JVS(1213)*X(121)-JVS(1214)*X(122)&
             &-JVS(1215)*X(123)-JVS(1216)*X(124)-JVS(1217)*X(125)-JVS(1218)*X(126)-JVS(1219)*X(127)-JVS(1220)*X(128)&
             &-JVS(1221)*X(129)-JVS(1222)*X(130)-JVS(1223)*X(131)-JVS(1224)*X(132)-JVS(1225)*X(133)-JVS(1226)*X(134)&
             &-JVS(1227)*X(135)
  X(137) = X(137)-JVS(1239)*X(74)-JVS(1240)*X(77)-JVS(1241)*X(83)-JVS(1242)*X(91)-JVS(1243)*X(92)-JVS(1244)*X(93)&
             &-JVS(1245)*X(102)-JVS(1246)*X(106)-JVS(1247)*X(112)-JVS(1248)*X(115)-JVS(1249)*X(118)-JVS(1250)*X(119)&
             &-JVS(1251)*X(122)-JVS(1252)*X(123)-JVS(1253)*X(124)-JVS(1254)*X(133)-JVS(1255)*X(134)-JVS(1256)*X(135)&
             &-JVS(1257)*X(136)
  X(138) = X(138)-JVS(1268)*X(47)-JVS(1269)*X(78)-JVS(1270)*X(79)-JVS(1271)*X(80)-JVS(1272)*X(82)-JVS(1273)*X(84)&
             &-JVS(1274)*X(85)-JVS(1275)*X(87)-JVS(1276)*X(89)-JVS(1277)*X(90)-JVS(1278)*X(91)-JVS(1279)*X(92)-JVS(1280)&
             &*X(93)-JVS(1281)*X(94)-JVS(1282)*X(96)-JVS(1283)*X(97)-JVS(1284)*X(99)-JVS(1285)*X(101)-JVS(1286)*X(103)&
             &-JVS(1287)*X(104)-JVS(1288)*X(105)-JVS(1289)*X(106)-JVS(1290)*X(107)-JVS(1291)*X(108)-JVS(1292)*X(109)&
             &-JVS(1293)*X(110)-JVS(1294)*X(111)-JVS(1295)*X(112)-JVS(1296)*X(113)-JVS(1297)*X(115)-JVS(1298)*X(116)&
             &-JVS(1299)*X(117)-JVS(1300)*X(118)-JVS(1301)*X(119)-JVS(1302)*X(120)-JVS(1303)*X(122)-JVS(1304)*X(123)&
             &-JVS(1305)*X(124)-JVS(1306)*X(126)-JVS(1307)*X(127)-JVS(1308)*X(128)-JVS(1309)*X(130)-JVS(1310)*X(132)&
             &-JVS(1311)*X(133)-JVS(1312)*X(134)-JVS(1313)*X(135)-JVS(1314)*X(136)-JVS(1315)*X(137)
  X(139) = X(139)-JVS(1325)*X(44)-JVS(1326)*X(46)-JVS(1327)*X(59)-JVS(1328)*X(68)-JVS(1329)*X(69)-JVS(1330)*X(83)&
             &-JVS(1331)*X(86)-JVS(1332)*X(88)-JVS(1333)*X(91)-JVS(1334)*X(96)-JVS(1335)*X(99)-JVS(1336)*X(101)-JVS(1337)&
             &*X(105)-JVS(1338)*X(106)-JVS(1339)*X(107)-JVS(1340)*X(108)-JVS(1341)*X(109)-JVS(1342)*X(110)-JVS(1343)*X(111)&
             &-JVS(1344)*X(112)-JVS(1345)*X(113)-JVS(1346)*X(115)-JVS(1347)*X(116)-JVS(1348)*X(117)-JVS(1349)*X(118)&
             &-JVS(1350)*X(119)-JVS(1351)*X(120)-JVS(1352)*X(121)-JVS(1353)*X(122)-JVS(1354)*X(123)-JVS(1355)*X(124)&
             &-JVS(1356)*X(126)-JVS(1357)*X(127)-JVS(1358)*X(128)-JVS(1359)*X(129)-JVS(1360)*X(130)-JVS(1361)*X(131)&
             &-JVS(1362)*X(132)-JVS(1363)*X(133)-JVS(1364)*X(134)-JVS(1365)*X(135)-JVS(1366)*X(136)-JVS(1367)*X(137)&
             &-JVS(1368)*X(138)
  X(140) = X(140)-JVS(1377)*X(70)-JVS(1378)*X(84)-JVS(1379)*X(91)-JVS(1380)*X(120)-JVS(1381)*X(122)-JVS(1382)*X(123)&
             &-JVS(1383)*X(124)-JVS(1384)*X(136)-JVS(1385)*X(137)-JVS(1386)*X(138)-JVS(1387)*X(139)
  X(141) = X(141)-JVS(1395)*X(39)-JVS(1396)*X(40)-JVS(1397)*X(42)-JVS(1398)*X(43)-JVS(1399)*X(44)-JVS(1400)*X(47)&
             &-JVS(1401)*X(48)-JVS(1402)*X(55)-JVS(1403)*X(61)-JVS(1404)*X(62)-JVS(1405)*X(63)-JVS(1406)*X(75)-JVS(1407)&
             &*X(76)-JVS(1408)*X(77)-JVS(1409)*X(78)-JVS(1410)*X(79)-JVS(1411)*X(80)-JVS(1412)*X(82)-JVS(1413)*X(83)&
             &-JVS(1414)*X(84)-JVS(1415)*X(85)-JVS(1416)*X(86)-JVS(1417)*X(87)-JVS(1418)*X(89)-JVS(1419)*X(90)-JVS(1420)&
             &*X(91)-JVS(1421)*X(92)-JVS(1422)*X(93)-JVS(1423)*X(94)-JVS(1424)*X(95)-JVS(1425)*X(96)-JVS(1426)*X(97)&
             &-JVS(1427)*X(98)-JVS(1428)*X(99)-JVS(1429)*X(100)-JVS(1430)*X(101)-JVS(1431)*X(102)-JVS(1432)*X(103)-JVS(1433)&
             &*X(104)-JVS(1434)*X(105)-JVS(1435)*X(106)-JVS(1436)*X(107)-JVS(1437)*X(108)-JVS(1438)*X(109)-JVS(1439)*X(110)&
             &-JVS(1440)*X(111)-JVS(1441)*X(112)-JVS(1442)*X(113)-JVS(1443)*X(114)-JVS(1444)*X(115)-JVS(1445)*X(116)&
             &-JVS(1446)*X(117)-JVS(1447)*X(118)-JVS(1448)*X(119)-JVS(1449)*X(120)-JVS(1450)*X(121)-JVS(1451)*X(122)&
             &-JVS(1452)*X(123)-JVS(1453)*X(124)-JVS(1454)*X(125)-JVS(1455)*X(126)-JVS(1456)*X(127)-JVS(1457)*X(128)&
             &-JVS(1458)*X(129)-JVS(1459)*X(130)-JVS(1460)*X(131)-JVS(1461)*X(132)-JVS(1462)*X(133)-JVS(1463)*X(134)&
             &-JVS(1464)*X(135)-JVS(1465)*X(136)-JVS(1466)*X(137)-JVS(1467)*X(138)-JVS(1468)*X(139)-JVS(1469)*X(140)
  X(142) = X(142)-JVS(1476)*X(70)-JVS(1477)*X(91)-JVS(1478)*X(104)-JVS(1479)*X(120)-JVS(1480)*X(122)-JVS(1481)*X(123)&
             &-JVS(1482)*X(124)-JVS(1483)*X(136)-JVS(1484)*X(137)-JVS(1485)*X(138)-JVS(1486)*X(139)-JVS(1487)*X(140)&
             &-JVS(1488)*X(141)
  X(143) = X(143)-JVS(1494)*X(36)-JVS(1495)*X(37)-JVS(1496)*X(38)-JVS(1497)*X(41)-JVS(1498)*X(45)-JVS(1499)*X(46)&
             &-JVS(1500)*X(47)-JVS(1501)*X(49)-JVS(1502)*X(50)-JVS(1503)*X(51)-JVS(1504)*X(52)-JVS(1505)*X(53)-JVS(1506)&
             &*X(54)-JVS(1507)*X(55)-JVS(1508)*X(56)-JVS(1509)*X(57)-JVS(1510)*X(58)-JVS(1511)*X(59)-JVS(1512)*X(60)&
             &-JVS(1513)*X(61)-JVS(1514)*X(62)-JVS(1515)*X(63)-JVS(1516)*X(64)-JVS(1517)*X(65)-JVS(1518)*X(66)-JVS(1519)&
             &*X(67)-JVS(1520)*X(68)-JVS(1521)*X(69)-JVS(1522)*X(70)-JVS(1523)*X(71)-JVS(1524)*X(72)-JVS(1525)*X(73)&
             &-JVS(1526)*X(74)-JVS(1527)*X(75)-JVS(1528)*X(76)-JVS(1529)*X(77)-JVS(1530)*X(78)-JVS(1531)*X(79)-JVS(1532)&
             &*X(80)-JVS(1533)*X(81)-JVS(1534)*X(82)-JVS(1535)*X(83)-JVS(1536)*X(84)-JVS(1537)*X(85)-JVS(1538)*X(86)&
             &-JVS(1539)*X(87)-JVS(1540)*X(88)-JVS(1541)*X(89)-JVS(1542)*X(90)-JVS(1543)*X(91)-JVS(1544)*X(92)-JVS(1545)&
             &*X(93)-JVS(1546)*X(94)-JVS(1547)*X(95)-JVS(1548)*X(96)-JVS(1549)*X(97)-JVS(1550)*X(98)-JVS(1551)*X(99)&
             &-JVS(1552)*X(100)-JVS(1553)*X(101)-JVS(1554)*X(102)-JVS(1555)*X(103)-JVS(1556)*X(104)-JVS(1557)*X(105)&
             &-JVS(1558)*X(106)-JVS(1559)*X(107)-JVS(1560)*X(108)-JVS(1561)*X(109)-JVS(1562)*X(110)-JVS(1563)*X(111)&
             &-JVS(1564)*X(112)-JVS(1565)*X(113)-JVS(1566)*X(114)-JVS(1567)*X(115)-JVS(1568)*X(116)-JVS(1569)*X(117)&
             &-JVS(1570)*X(118)-JVS(1571)*X(119)-JVS(1572)*X(120)-JVS(1573)*X(121)-JVS(1574)*X(122)-JVS(1575)*X(123)&
             &-JVS(1576)*X(124)-JVS(1577)*X(125)-JVS(1578)*X(126)-JVS(1579)*X(127)-JVS(1580)*X(128)-JVS(1581)*X(129)&
             &-JVS(1582)*X(130)-JVS(1583)*X(131)-JVS(1584)*X(132)-JVS(1585)*X(133)-JVS(1586)*X(134)-JVS(1587)*X(135)&
             &-JVS(1588)*X(136)-JVS(1589)*X(137)-JVS(1590)*X(138)-JVS(1591)*X(139)-JVS(1592)*X(140)-JVS(1593)*X(141)&
             &-JVS(1594)*X(142)
  X(144) = X(144)-JVS(1599)*X(48)-JVS(1600)*X(49)-JVS(1601)*X(50)-JVS(1602)*X(52)-JVS(1603)*X(54)-JVS(1604)*X(55)&
             &-JVS(1605)*X(60)-JVS(1606)*X(61)-JVS(1607)*X(65)-JVS(1608)*X(66)-JVS(1609)*X(67)-JVS(1610)*X(69)-JVS(1611)&
             &*X(70)-JVS(1612)*X(71)-JVS(1613)*X(72)-JVS(1614)*X(73)-JVS(1615)*X(74)-JVS(1616)*X(75)-JVS(1617)*X(76)&
             &-JVS(1618)*X(77)-JVS(1619)*X(78)-JVS(1620)*X(79)-JVS(1621)*X(80)-JVS(1622)*X(81)-JVS(1623)*X(82)-JVS(1624)&
             &*X(84)-JVS(1625)*X(85)-JVS(1626)*X(87)-JVS(1627)*X(89)-JVS(1628)*X(90)-JVS(1629)*X(91)-JVS(1630)*X(92)&
             &-JVS(1631)*X(93)-JVS(1632)*X(94)-JVS(1633)*X(96)-JVS(1634)*X(97)-JVS(1635)*X(99)-JVS(1636)*X(100)-JVS(1637)&
             &*X(101)-JVS(1638)*X(102)-JVS(1639)*X(103)-JVS(1640)*X(104)-JVS(1641)*X(105)-JVS(1642)*X(106)-JVS(1643)*X(107)&
             &-JVS(1644)*X(108)-JVS(1645)*X(109)-JVS(1646)*X(110)-JVS(1647)*X(111)-JVS(1648)*X(112)-JVS(1649)*X(113)&
             &-JVS(1650)*X(114)-JVS(1651)*X(115)-JVS(1652)*X(116)-JVS(1653)*X(117)-JVS(1654)*X(118)-JVS(1655)*X(119)&
             &-JVS(1656)*X(120)-JVS(1657)*X(121)-JVS(1658)*X(122)-JVS(1659)*X(123)-JVS(1660)*X(124)-JVS(1661)*X(125)&
             &-JVS(1662)*X(126)-JVS(1663)*X(127)-JVS(1664)*X(128)-JVS(1665)*X(129)-JVS(1666)*X(130)-JVS(1667)*X(131)&
             &-JVS(1668)*X(132)-JVS(1669)*X(133)-JVS(1670)*X(134)-JVS(1671)*X(135)-JVS(1672)*X(136)-JVS(1673)*X(137)&
             &-JVS(1674)*X(138)-JVS(1675)*X(139)-JVS(1676)*X(140)-JVS(1677)*X(141)-JVS(1678)*X(142)-JVS(1679)*X(143)
  X(145) = X(145)-JVS(1683)*X(44)-JVS(1684)*X(46)-JVS(1685)*X(48)-JVS(1686)*X(51)-JVS(1687)*X(52)-JVS(1688)*X(88)&
             &-JVS(1689)*X(89)-JVS(1690)*X(91)-JVS(1691)*X(92)-JVS(1692)*X(95)-JVS(1693)*X(96)-JVS(1694)*X(97)-JVS(1695)&
             &*X(99)-JVS(1696)*X(101)-JVS(1697)*X(102)-JVS(1698)*X(105)-JVS(1699)*X(107)-JVS(1700)*X(108)-JVS(1701)*X(109)&
             &-JVS(1702)*X(110)-JVS(1703)*X(111)-JVS(1704)*X(112)-JVS(1705)*X(113)-JVS(1706)*X(115)-JVS(1707)*X(116)&
             &-JVS(1708)*X(117)-JVS(1709)*X(118)-JVS(1710)*X(119)-JVS(1711)*X(120)-JVS(1712)*X(122)-JVS(1713)*X(123)&
             &-JVS(1714)*X(124)-JVS(1715)*X(126)-JVS(1716)*X(127)-JVS(1717)*X(128)-JVS(1718)*X(129)-JVS(1719)*X(130)&
             &-JVS(1720)*X(131)-JVS(1721)*X(132)-JVS(1722)*X(133)-JVS(1723)*X(134)-JVS(1724)*X(135)-JVS(1725)*X(136)&
             &-JVS(1726)*X(137)-JVS(1727)*X(138)-JVS(1728)*X(139)-JVS(1729)*X(140)-JVS(1730)*X(141)-JVS(1731)*X(142)&
             &-JVS(1732)*X(143)-JVS(1733)*X(144)
  X(146) = X(146)-JVS(1736)*X(43)-JVS(1737)*X(44)-JVS(1738)*X(45)-JVS(1739)*X(48)-JVS(1740)*X(51)-JVS(1741)*X(55)&
             &-JVS(1742)*X(62)-JVS(1743)*X(64)-JVS(1744)*X(79)-JVS(1745)*X(83)-JVS(1746)*X(84)-JVS(1747)*X(85)-JVS(1748)&
             &*X(86)-JVS(1749)*X(87)-JVS(1750)*X(90)-JVS(1751)*X(91)-JVS(1752)*X(92)-JVS(1753)*X(93)-JVS(1754)*X(94)&
             &-JVS(1755)*X(96)-JVS(1756)*X(97)-JVS(1757)*X(98)-JVS(1758)*X(100)-JVS(1759)*X(101)-JVS(1760)*X(103)-JVS(1761)&
             &*X(104)-JVS(1762)*X(106)-JVS(1763)*X(111)-JVS(1764)*X(112)-JVS(1765)*X(114)-JVS(1766)*X(118)-JVS(1767)*X(119)&
             &-JVS(1768)*X(120)-JVS(1769)*X(121)-JVS(1770)*X(122)-JVS(1771)*X(123)-JVS(1772)*X(124)-JVS(1773)*X(125)&
             &-JVS(1774)*X(126)-JVS(1775)*X(127)-JVS(1776)*X(128)-JVS(1777)*X(129)-JVS(1778)*X(130)-JVS(1779)*X(131)&
             &-JVS(1780)*X(132)-JVS(1781)*X(133)-JVS(1782)*X(134)-JVS(1783)*X(135)-JVS(1784)*X(136)-JVS(1785)*X(137)&
             &-JVS(1786)*X(138)-JVS(1787)*X(139)-JVS(1788)*X(140)-JVS(1789)*X(141)-JVS(1790)*X(142)-JVS(1791)*X(143)&
             &-JVS(1792)*X(144)-JVS(1793)*X(145)
  X(146) = X(146)/JVS(1794)
  X(145) = (X(145)-JVS(1735)*X(146))/(JVS(1734))
  X(144) = (X(144)-JVS(1681)*X(145)-JVS(1682)*X(146))/(JVS(1680))
  X(143) = (X(143)-JVS(1596)*X(144)-JVS(1597)*X(145)-JVS(1598)*X(146))/(JVS(1595))
  X(142) = (X(142)-JVS(1490)*X(143)-JVS(1491)*X(144)-JVS(1492)*X(145)-JVS(1493)*X(146))/(JVS(1489))
  X(141) = (X(141)-JVS(1471)*X(142)-JVS(1472)*X(143)-JVS(1473)*X(144)-JVS(1474)*X(145)-JVS(1475)*X(146))/(JVS(1470))
  X(140) = (X(140)-JVS(1389)*X(141)-JVS(1390)*X(142)-JVS(1391)*X(143)-JVS(1392)*X(144)-JVS(1393)*X(145)-JVS(1394)&
             &*X(146))/(JVS(1388))
  X(139) = (X(139)-JVS(1370)*X(140)-JVS(1371)*X(141)-JVS(1372)*X(142)-JVS(1373)*X(143)-JVS(1374)*X(144)-JVS(1375)*X(145)&
             &-JVS(1376)*X(146))/(JVS(1369))
  X(138) = (X(138)-JVS(1317)*X(139)-JVS(1318)*X(140)-JVS(1319)*X(141)-JVS(1320)*X(142)-JVS(1321)*X(143)-JVS(1322)*X(144)&
             &-JVS(1323)*X(145)-JVS(1324)*X(146))/(JVS(1316))
  X(137) = (X(137)-JVS(1259)*X(138)-JVS(1260)*X(139)-JVS(1261)*X(140)-JVS(1262)*X(141)-JVS(1263)*X(142)-JVS(1264)*X(143)&
             &-JVS(1265)*X(144)-JVS(1266)*X(145)-JVS(1267)*X(146))/(JVS(1258))
  X(136) = (X(136)-JVS(1229)*X(137)-JVS(1230)*X(138)-JVS(1231)*X(139)-JVS(1232)*X(140)-JVS(1233)*X(141)-JVS(1234)*X(142)&
             &-JVS(1235)*X(143)-JVS(1236)*X(144)-JVS(1237)*X(145)-JVS(1238)*X(146))/(JVS(1228))
  X(135) = (X(135)-JVS(1173)*X(136)-JVS(1174)*X(137)-JVS(1175)*X(138)-JVS(1176)*X(139)-JVS(1177)*X(140)-JVS(1178)*X(141)&
             &-JVS(1179)*X(142)-JVS(1180)*X(143)-JVS(1181)*X(144)-JVS(1182)*X(145)-JVS(1183)*X(146))/(JVS(1172))
  X(134) = (X(134)-JVS(1139)*X(135)-JVS(1140)*X(136)-JVS(1141)*X(137)-JVS(1142)*X(138)-JVS(1143)*X(139)-JVS(1144)*X(140)&
             &-JVS(1145)*X(141)-JVS(1146)*X(142)-JVS(1147)*X(143)-JVS(1148)*X(144)-JVS(1149)*X(145)-JVS(1150)*X(146))&
             &/(JVS(1138))
  X(133) = (X(133)-JVS(1103)*X(134)-JVS(1104)*X(135)-JVS(1105)*X(136)-JVS(1106)*X(137)-JVS(1107)*X(138)-JVS(1108)*X(139)&
             &-JVS(1109)*X(140)-JVS(1110)*X(141)-JVS(1111)*X(142)-JVS(1112)*X(143)-JVS(1113)*X(144)-JVS(1114)*X(145)&
             &-JVS(1115)*X(146))/(JVS(1102))
  X(132) = (X(132)-JVS(1061)*X(134)-JVS(1062)*X(135)-JVS(1063)*X(137)-JVS(1064)*X(138)-JVS(1065)*X(139)-JVS(1066)*X(141)&
             &-JVS(1067)*X(142)-JVS(1068)*X(143)-JVS(1069)*X(144)-JVS(1070)*X(145)-JVS(1071)*X(146))/(JVS(1060))
  X(131) = (X(131)-JVS(1032)*X(132)-JVS(1033)*X(133)-JVS(1034)*X(134)-JVS(1035)*X(135)-JVS(1036)*X(136)-JVS(1037)*X(137)&
             &-JVS(1038)*X(138)-JVS(1039)*X(139)-JVS(1040)*X(140)-JVS(1041)*X(141)-JVS(1042)*X(142)-JVS(1043)*X(143)&
             &-JVS(1044)*X(144)-JVS(1045)*X(145)-JVS(1046)*X(146))/(JVS(1031))
  X(130) = (X(130)-JVS(1003)*X(132)-JVS(1004)*X(137)-JVS(1005)*X(138)-JVS(1006)*X(139)-JVS(1007)*X(142)-JVS(1008)*X(143)&
             &-JVS(1009)*X(144)-JVS(1010)*X(145)-JVS(1011)*X(146))/(JVS(1002))
  X(129) = (X(129)-JVS(977)*X(130)-JVS(978)*X(132)-JVS(979)*X(134)-JVS(980)*X(135)-JVS(981)*X(136)-JVS(982)*X(137)&
             &-JVS(983)*X(138)-JVS(984)*X(139)-JVS(985)*X(140)-JVS(986)*X(141)-JVS(987)*X(142)-JVS(988)*X(143)-JVS(989)&
             &*X(144)-JVS(990)*X(145)-JVS(991)*X(146))/(JVS(976))
  X(128) = (X(128)-JVS(942)*X(137)-JVS(943)*X(138)-JVS(944)*X(139)-JVS(945)*X(142)-JVS(946)*X(143)-JVS(947)*X(144)&
             &-JVS(948)*X(145)-JVS(949)*X(146))/(JVS(941))
  X(127) = (X(127)-JVS(926)*X(128)-JVS(927)*X(137)-JVS(928)*X(138)-JVS(929)*X(139)-JVS(930)*X(142)-JVS(931)*X(143)&
             &-JVS(932)*X(144)-JVS(933)*X(145)-JVS(934)*X(146))/(JVS(925))
  X(126) = (X(126)-JVS(910)*X(136)-JVS(911)*X(137)-JVS(912)*X(138)-JVS(913)*X(139)-JVS(914)*X(140)-JVS(915)*X(141)&
             &-JVS(916)*X(142)-JVS(917)*X(143)-JVS(918)*X(144)-JVS(919)*X(145)-JVS(920)*X(146))/(JVS(909))
  X(125) = (X(125)-JVS(884)*X(126)-JVS(885)*X(127)-JVS(886)*X(128)-JVS(887)*X(129)-JVS(888)*X(130)-JVS(889)*X(131)&
             &-JVS(890)*X(132)-JVS(891)*X(133)-JVS(892)*X(134)-JVS(893)*X(135)-JVS(894)*X(136)-JVS(895)*X(137)-JVS(896)&
             &*X(138)-JVS(897)*X(139)-JVS(898)*X(140)-JVS(899)*X(141)-JVS(900)*X(142)-JVS(901)*X(143)-JVS(902)*X(144)&
             &-JVS(903)*X(145)-JVS(904)*X(146))/(JVS(883))
  X(124) = (X(124)-JVS(830)*X(137)-JVS(831)*X(138)-JVS(832)*X(139)-JVS(833)*X(143)-JVS(834)*X(144)-JVS(835)*X(145)&
             &-JVS(836)*X(146))/(JVS(829))
  X(123) = (X(123)-JVS(817)*X(137)-JVS(818)*X(138)-JVS(819)*X(139)-JVS(820)*X(143)-JVS(821)*X(144)-JVS(822)*X(145)&
             &-JVS(823)*X(146))/(JVS(816))
  X(122) = (X(122)-JVS(804)*X(137)-JVS(805)*X(138)-JVS(806)*X(139)-JVS(807)*X(143)-JVS(808)*X(144)-JVS(809)*X(145)&
             &-JVS(810)*X(146))/(JVS(803))
  X(121) = (X(121)-JVS(779)*X(122)-JVS(780)*X(123)-JVS(781)*X(124)-JVS(782)*X(126)-JVS(783)*X(127)-JVS(784)*X(128)&
             &-JVS(785)*X(129)-JVS(786)*X(130)-JVS(787)*X(132)-JVS(788)*X(134)-JVS(789)*X(135)-JVS(790)*X(136)-JVS(791)&
             &*X(137)-JVS(792)*X(138)-JVS(793)*X(139)-JVS(794)*X(140)-JVS(795)*X(141)-JVS(796)*X(142)-JVS(797)*X(143)&
             &-JVS(798)*X(144)-JVS(799)*X(145)-JVS(800)*X(146))/(JVS(778))
  X(120) = (X(120)-JVS(750)*X(124)-JVS(751)*X(136)-JVS(752)*X(137)-JVS(753)*X(138)-JVS(754)*X(140)-JVS(755)*X(141)&
             &-JVS(756)*X(142)-JVS(757)*X(143)-JVS(758)*X(146))/(JVS(749))
  X(119) = (X(119)-JVS(729)*X(123)-JVS(730)*X(137)-JVS(731)*X(138)-JVS(732)*X(139)-JVS(733)*X(141)-JVS(734)*X(142)&
             &-JVS(735)*X(143)-JVS(736)*X(144)-JVS(737)*X(145)-JVS(738)*X(146))/(JVS(728))
  X(118) = (X(118)-JVS(717)*X(123)-JVS(718)*X(137)-JVS(719)*X(138)-JVS(720)*X(139)-JVS(721)*X(143)-JVS(722)*X(145)&
             &-JVS(723)*X(146))/(JVS(716))
  X(117) = (X(117)-JVS(706)*X(127)-JVS(707)*X(128)-JVS(708)*X(132)-JVS(709)*X(138)-JVS(710)*X(139)-JVS(711)*X(142)&
             &-JVS(712)*X(143)-JVS(713)*X(144)-JVS(714)*X(145)-JVS(715)*X(146))/(JVS(705))
  X(116) = (X(116)-JVS(693)*X(130)-JVS(694)*X(132)-JVS(695)*X(134)-JVS(696)*X(135)-JVS(697)*X(138)-JVS(698)*X(139)&
             &-JVS(699)*X(143)-JVS(700)*X(144)-JVS(701)*X(145)-JVS(702)*X(146))/(JVS(692))
  X(115) = (X(115)-JVS(677)*X(118)-JVS(678)*X(133)-JVS(679)*X(135)-JVS(680)*X(138)-JVS(681)*X(139)-JVS(682)*X(140)&
             &-JVS(683)*X(141)-JVS(684)*X(143)-JVS(685)*X(144)-JVS(686)*X(145)-JVS(687)*X(146))/(JVS(676))
  X(114) = (X(114)-JVS(657)*X(120)-JVS(658)*X(122)-JVS(659)*X(123)-JVS(660)*X(124)-JVS(661)*X(127)-JVS(662)*X(128)&
             &-JVS(663)*X(136)-JVS(664)*X(137)-JVS(665)*X(138)-JVS(666)*X(139)-JVS(667)*X(140)-JVS(668)*X(141)-JVS(669)&
             &*X(143)-JVS(670)*X(144)-JVS(671)*X(145)-JVS(672)*X(146))/(JVS(656))
  X(113) = (X(113)-JVS(635)*X(115)-JVS(636)*X(130)-JVS(637)*X(132)-JVS(638)*X(133)-JVS(639)*X(134)-JVS(640)*X(135)&
             &-JVS(641)*X(138)-JVS(642)*X(139)-JVS(643)*X(143)-JVS(644)*X(144)-JVS(645)*X(145)-JVS(646)*X(146))/(JVS(634))
  X(112) = (X(112)-JVS(626)*X(124)-JVS(627)*X(138)-JVS(628)*X(139)-JVS(629)*X(143)-JVS(630)*X(144)-JVS(631)*X(145))&
             &/(JVS(625))
  X(111) = (X(111)-JVS(616)*X(137)-JVS(617)*X(138)-JVS(618)*X(139)-JVS(619)*X(140)-JVS(620)*X(143)-JVS(621)*X(144)&
             &-JVS(622)*X(145)-JVS(623)*X(146))/(JVS(615))
  X(110) = (X(110)-JVS(606)*X(135)-JVS(607)*X(138)-JVS(608)*X(139)-JVS(609)*X(143)-JVS(610)*X(144)-JVS(611)*X(145)&
             &-JVS(612)*X(146))/(JVS(605))
  X(109) = (X(109)-JVS(597)*X(137)-JVS(598)*X(138)-JVS(599)*X(139)-JVS(600)*X(140)-JVS(601)*X(143)-JVS(602)*X(144)&
             &-JVS(603)*X(145)-JVS(604)*X(146))/(JVS(596))
  X(108) = (X(108)-JVS(586)*X(130)-JVS(587)*X(132)-JVS(588)*X(134)-JVS(589)*X(138)-JVS(590)*X(139)-JVS(591)*X(143)&
             &-JVS(592)*X(144)-JVS(593)*X(145))/(JVS(585))
  X(107) = (X(107)-JVS(577)*X(138)-JVS(578)*X(139)-JVS(579)*X(142)-JVS(580)*X(144)-JVS(581)*X(145)-JVS(582)*X(146))&
             &/(JVS(576))
  X(106) = (X(106)-JVS(570)*X(118)-JVS(571)*X(138)-JVS(572)*X(141)-JVS(573)*X(143)-JVS(574)*X(144)-JVS(575)*X(146))&
             &/(JVS(569))
  X(105) = (X(105)-JVS(561)*X(137)-JVS(562)*X(138)-JVS(563)*X(139)-JVS(564)*X(140)-JVS(565)*X(143)-JVS(566)*X(144)&
             &-JVS(567)*X(145)-JVS(568)*X(146))/(JVS(560))
  X(104) = (X(104)-JVS(551)*X(120)-JVS(552)*X(136)-JVS(553)*X(137)-JVS(554)*X(138)-JVS(555)*X(141)-JVS(556)*X(143)&
             &-JVS(557)*X(146))/(JVS(550))
  X(103) = (X(103)-JVS(541)*X(122)-JVS(542)*X(126)-JVS(543)*X(127)-JVS(544)*X(128)-JVS(545)*X(137)-JVS(546)*X(138)&
             &-JVS(547)*X(143)-JVS(548)*X(144))/(JVS(540))
  X(102) = (X(102)-JVS(533)*X(122)-JVS(534)*X(123)-JVS(535)*X(137)-JVS(536)*X(138)-JVS(537)*X(143)-JVS(538)*X(145))&
             &/(JVS(532))
  X(101) = (X(101)-JVS(526)*X(119)-JVS(527)*X(138)-JVS(528)*X(139)-JVS(529)*X(143)-JVS(530)*X(144)-JVS(531)*X(145))&
             &/(JVS(525))
  X(100) = (X(100)-JVS(513)*X(112)-JVS(514)*X(114)-JVS(515)*X(118)-JVS(516)*X(122)-JVS(517)*X(137)-JVS(518)*X(138)&
             &-JVS(519)*X(139)-JVS(520)*X(143)-JVS(521)*X(144)-JVS(522)*X(145)-JVS(523)*X(146))/(JVS(512))
  X(99) = (X(99)-JVS(500)*X(130)-JVS(501)*X(132)-JVS(502)*X(134)-JVS(503)*X(138)-JVS(504)*X(139)-JVS(505)*X(143)&
            &-JVS(506)*X(144)-JVS(507)*X(145))/(JVS(499))
  X(98) = (X(98)-JVS(476)*X(100)-JVS(477)*X(101)-JVS(478)*X(112)-JVS(479)*X(118)-JVS(480)*X(121)-JVS(481)*X(122)&
            &-JVS(482)*X(125)-JVS(483)*X(127)-JVS(484)*X(128)-JVS(485)*X(131)-JVS(486)*X(133)-JVS(487)*X(134)-JVS(488)&
            &*X(135)-JVS(489)*X(137)-JVS(490)*X(138)-JVS(491)*X(139)-JVS(492)*X(141)-JVS(493)*X(142)-JVS(494)*X(143)&
            &-JVS(495)*X(144)-JVS(496)*X(146))/(JVS(475))
  X(97) = (X(97)-JVS(462)*X(118)-JVS(463)*X(138)-JVS(464)*X(144)-JVS(465)*X(145)-JVS(466)*X(146))/(JVS(461))
  X(96) = (X(96)-JVS(455)*X(120)-JVS(456)*X(136)-JVS(457)*X(137)-JVS(458)*X(138)-JVS(459)*X(140)-JVS(460)*X(141))&
            &/(JVS(454))
  X(95) = (X(95)-JVS(444)*X(102)-JVS(445)*X(107)-JVS(446)*X(109)-JVS(447)*X(128)-JVS(448)*X(137)-JVS(449)*X(138)&
            &-JVS(450)*X(139)-JVS(451)*X(143)-JVS(452)*X(144)-JVS(453)*X(145))/(JVS(443))
  X(94) = (X(94)-JVS(435)*X(120)-JVS(436)*X(124)-JVS(437)*X(136)-JVS(438)*X(137)-JVS(439)*X(138)-JVS(440)*X(141)&
            &-JVS(441)*X(142))/(JVS(434))
  X(93) = (X(93)-JVS(427)*X(134)-JVS(428)*X(137)-JVS(429)*X(138)-JVS(430)*X(141)-JVS(431)*X(143)-JVS(432)*X(144)&
            &-JVS(433)*X(146))/(JVS(426))
  X(92) = (X(92)-JVS(421)*X(137)-JVS(422)*X(140)-JVS(423)*X(143)-JVS(424)*X(146))/(JVS(420))
  X(91) = (X(91)-JVS(416)*X(137)-JVS(417)*X(143)-JVS(418)*X(146))/(JVS(415))
  X(90) = (X(90)-JVS(407)*X(91)-JVS(408)*X(92)-JVS(409)*X(120)-JVS(410)*X(137)-JVS(411)*X(138)-JVS(412)*X(140)-JVS(413)&
            &*X(141)-JVS(414)*X(142))/(JVS(406))
  X(89) = (X(89)-JVS(401)*X(118)-JVS(402)*X(138)-JVS(403)*X(143)-JVS(404)*X(144)-JVS(405)*X(145))/(JVS(400))
  X(88) = (X(88)-JVS(391)*X(108)-JVS(392)*X(130)-JVS(393)*X(132)-JVS(394)*X(134)-JVS(395)*X(138)-JVS(396)*X(139)&
            &-JVS(397)*X(143)-JVS(398)*X(144)-JVS(399)*X(145))/(JVS(390))
  X(87) = (X(87)-JVS(383)*X(120)-JVS(384)*X(124)-JVS(385)*X(136)-JVS(386)*X(137)-JVS(387)*X(138)-JVS(388)*X(141))&
            &/(JVS(382))
  X(86) = (X(86)-JVS(375)*X(111)-JVS(376)*X(122)-JVS(377)*X(127)-JVS(378)*X(137)-JVS(379)*X(138)-JVS(380)*X(143)&
            &-JVS(381)*X(144))/(JVS(374))
  X(85) = (X(85)-JVS(367)*X(92)-JVS(368)*X(120)-JVS(369)*X(136)-JVS(370)*X(137)-JVS(371)*X(138)-JVS(372)*X(141))&
            &/(JVS(366))
  X(84) = (X(84)-JVS(360)*X(91)-JVS(361)*X(120)-JVS(362)*X(136)-JVS(363)*X(137)-JVS(364)*X(138)-JVS(365)*X(141))&
            &/(JVS(359))
  X(83) = (X(83)-JVS(354)*X(106)-JVS(355)*X(119)-JVS(356)*X(137)-JVS(357)*X(141)-JVS(358)*X(143))/(JVS(353))
  X(82) = (X(82)-JVS(349)*X(122)-JVS(350)*X(138)-JVS(351)*X(139)-JVS(352)*X(144))/(JVS(348))
  X(81) = (X(81)-JVS(334)*X(99)-JVS(335)*X(101)-JVS(336)*X(105)-JVS(337)*X(108)-JVS(338)*X(109)-JVS(339)*X(110)-JVS(340)&
            &*X(111)-JVS(341)*X(112)-JVS(342)*X(117)-JVS(343)*X(122)-JVS(344)*X(126)-JVS(345)*X(130)-JVS(346)*X(143)&
            &-JVS(347)*X(145))/(JVS(333))
  X(80) = (X(80)-JVS(327)*X(112)-JVS(328)*X(137)-JVS(329)*X(138)-JVS(330)*X(139)-JVS(331)*X(143)-JVS(332)*X(144))&
            &/(JVS(326))
  X(79) = (X(79)-JVS(320)*X(103)-JVS(321)*X(138)-JVS(322)*X(141)-JVS(323)*X(143)-JVS(324)*X(144))/(JVS(319))
  X(78) = (X(78)-JVS(314)*X(122)-JVS(315)*X(138)-JVS(316)*X(143)-JVS(317)*X(144))/(JVS(313))
  X(77) = (X(77)-JVS(307)*X(122)-JVS(308)*X(137)-JVS(309)*X(138)-JVS(310)*X(143))/(JVS(306))
  X(76) = (X(76)-JVS(301)*X(107)-JVS(302)*X(127)-JVS(303)*X(128)-JVS(304)*X(143)-JVS(305)*X(144))/(JVS(300))
  X(75) = (X(75)-JVS(295)*X(89)-JVS(296)*X(97)-JVS(297)*X(123)-JVS(298)*X(143)-JVS(299)*X(144))/(JVS(294))
  X(74) = (X(74)-JVS(289)*X(112)-JVS(290)*X(137)-JVS(291)*X(138)-JVS(292)*X(139)-JVS(293)*X(143))/(JVS(288))
  X(73) = (X(73)-JVS(283)*X(112)-JVS(284)*X(138)-JVS(285)*X(139)-JVS(286)*X(143)-JVS(287)*X(144))/(JVS(282))
  X(72) = (X(72)-JVS(278)*X(112)-JVS(279)*X(143)-JVS(280)*X(144))/(JVS(277))
  X(71) = (X(71)-JVS(273)*X(104)-JVS(274)*X(126)-JVS(275)*X(143)-JVS(276)*X(144))/(JVS(272))
  X(70) = (X(70)-JVS(269)*X(122)-JVS(270)*X(143)-JVS(271)*X(144))/(JVS(268))
  X(69) = (X(69)-JVS(265)*X(111)-JVS(266)*X(143)-JVS(267)*X(144))/(JVS(264))
  X(68) = (X(68)-JVS(260)*X(110)-JVS(261)*X(116)-JVS(262)*X(143)-JVS(263)*X(144))/(JVS(259))
  X(67) = (X(67)-JVS(254)*X(80)-JVS(255)*X(82)-JVS(256)*X(130)-JVS(257)*X(143)-JVS(258)*X(144))/(JVS(253))
  X(66) = (X(66)-JVS(244)*X(84)-JVS(245)*X(85)-JVS(246)*X(87)-JVS(247)*X(90)-JVS(248)*X(94)-JVS(249)*X(96)-JVS(250)&
            &*X(104)-JVS(251)*X(143)-JVS(252)*X(144))/(JVS(243))
  X(65) = (X(65)-JVS(240)*X(105)-JVS(241)*X(143)-JVS(242)*X(144))/(JVS(239))
  X(64) = (X(64)-JVS(236)*X(134)-JVS(237)*X(143)-JVS(238)*X(146))/(JVS(235))
  X(63) = (X(63)-JVS(229)*X(102)-JVS(230)*X(128)-JVS(231)*X(137)-JVS(232)*X(138)-JVS(233)*X(143)-JVS(234)*X(144))&
            &/(JVS(228))
  X(62) = (X(62)-JVS(225)*X(93)-JVS(226)*X(134)-JVS(227)*X(141))/(JVS(224))
  X(61) = (X(61)-JVS(221)*X(109)-JVS(222)*X(143)-JVS(223)*X(144))/(JVS(220))
  X(60) = (X(60)-JVS(216)*X(106)-JVS(217)*X(115)-JVS(218)*X(143)-JVS(219)*X(144))/(JVS(215))
  X(59) = (X(59)-JVS(212)*X(119)-JVS(213)*X(143)-JVS(214)*X(144))/(JVS(211))
  X(58) = (X(58)-JVS(204)*X(64)-JVS(205)*X(88)-JVS(206)*X(125)-JVS(207)*X(131)-JVS(208)*X(134)-JVS(209)*X(143)-JVS(210)&
            &*X(144))/(JVS(203))
  X(57) = (X(57)-JVS(196)*X(58)-JVS(197)*X(62)-JVS(198)*X(93)-JVS(199)*X(134)-JVS(200)*X(143)-JVS(201)*X(144))&
            &/(JVS(195))
  X(56) = (X(56)-JVS(189)*X(57)-JVS(190)*X(62)-JVS(191)*X(93)-JVS(192)*X(143)-JVS(193)*X(144))/(JVS(188))
  X(55) = (X(55)-JVS(185)*X(141)-JVS(186)*X(143)-JVS(187)*X(144))/(JVS(184))
  X(54) = (X(54)-JVS(181)*X(113)-JVS(182)*X(143)-JVS(183)*X(144))/(JVS(180))
  X(53) = (X(53)-JVS(178)*X(134)-JVS(179)*X(143))/(JVS(177))
  X(52) = (X(52)-JVS(174)*X(143)-JVS(175)*X(144)-JVS(176)*X(145))/(JVS(173))
  X(51) = (X(51)-JVS(171)*X(143)-JVS(172)*X(146))/(JVS(170))
  X(50) = (X(50)-JVS(167)*X(108)-JVS(168)*X(143)-JVS(169)*X(144))/(JVS(166))
  X(49) = (X(49)-JVS(163)*X(99)-JVS(164)*X(143)-JVS(165)*X(144))/(JVS(162))
  X(48) = (X(48)-JVS(160)*X(141)-JVS(161)*X(145))/(JVS(159))
  X(47) = (X(47)-JVS(156)*X(138)-JVS(157)*X(141)-JVS(158)*X(143))/(JVS(155))
  X(46) = (X(46)-JVS(152)*X(139)-JVS(153)*X(143)-JVS(154)*X(144))/(JVS(151))
  X(45) = (X(45)-JVS(149)*X(143)-JVS(150)*X(146))/(JVS(148))
  X(44) = (X(44)-JVS(146)*X(139)-JVS(147)*X(141))/(JVS(145))
  X(43) = (X(43)-JVS(143)*X(141)-JVS(144)*X(146))/(JVS(142))
  X(42) = (X(42)-JVS(140)*X(79)-JVS(141)*X(141))/(JVS(139))
  X(41) = (X(41)-JVS(137)*X(70)-JVS(138)*X(143))/(JVS(136))
  X(40) = (X(40)-JVS(134)*X(134)-JVS(135)*X(141))/(JVS(133))
  X(39) = (X(39)-JVS(131)*X(115)-JVS(132)*X(141))/(JVS(130))
  X(38) = (X(38)-JVS(129)*X(143))/(JVS(128))
  X(37) = (X(37)-JVS(127)*X(143))/(JVS(126))
  X(36) = (X(36)-JVS(125)*X(143))/(JVS(124))
  X(35) = (X(35)-JVS(97)*X(39)-JVS(98)*X(41)-JVS(99)*X(42)-JVS(100)*X(43)-JVS(101)*X(44)-JVS(102)*X(56)-JVS(103)*X(57)&
            &-JVS(104)*X(58)-JVS(105)*X(62)-JVS(106)*X(66)-JVS(107)*X(70)-JVS(108)*X(77)-JVS(109)*X(83)-JVS(110)*X(86)&
            &-JVS(111)*X(88)-JVS(112)*X(95)-JVS(113)*X(98)-JVS(114)*X(102)-JVS(115)*X(103)-JVS(116)*X(125)-JVS(117)*X(129)&
            &-JVS(118)*X(131)-JVS(119)*X(132)-JVS(120)*X(137)-JVS(121)*X(140)-JVS(122)*X(141)-JVS(123)*X(142))/(JVS(96))
  X(34) = (X(34)-JVS(77)*X(63)-JVS(78)*X(71)-JVS(79)*X(74)-JVS(80)*X(78)-JVS(81)*X(79)-JVS(82)*X(80)-JVS(83)*X(101)&
            &-JVS(84)*X(107)-JVS(85)*X(114)-JVS(86)*X(119)-JVS(87)*X(122)-JVS(88)*X(129)-JVS(89)*X(136)-JVS(90)*X(137)&
            &-JVS(91)*X(138)-JVS(92)*X(139)-JVS(93)*X(143)-JVS(94)*X(144)-JVS(95)*X(145))/(JVS(76))
  X(33) = X(33)/JVS(75)
  X(32) = X(32)/JVS(74)
  X(31) = (X(31)-JVS(73)*X(95))/(JVS(72))
  X(30) = (X(30)-JVS(71)*X(86))/(JVS(70))
  X(29) = (X(29)-JVS(69)*X(103))/(JVS(68))
  X(28) = (X(28)-JVS(67)*X(41))/(JVS(66))
  X(27) = (X(27)-JVS(65)*X(70))/(JVS(64))
  X(26) = (X(26)-JVS(63)*X(77))/(JVS(62))
  X(25) = (X(25)-JVS(61)*X(102))/(JVS(60))
  X(24) = (X(24)-JVS(58)*X(86)-JVS(59)*X(143))/(JVS(57))
  X(23) = (X(23)-JVS(56)*X(57))/(JVS(55))
  X(22) = (X(22)-JVS(54)*X(62))/(JVS(53))
  X(21) = (X(21)-JVS(52)*X(58))/(JVS(51))
  X(20) = (X(20)-JVS(50)*X(56))/(JVS(49))
  X(19) = (X(19)-JVS(47)*X(91)-JVS(48)*X(143))/(JVS(46))
  X(18) = (X(18)-JVS(44)*X(51)-JVS(45)*X(143))/(JVS(43))
  X(17) = (X(17)-JVS(34)*X(84)-JVS(35)*X(85)-JVS(36)*X(87)-JVS(37)*X(90)-JVS(38)*X(94)-JVS(39)*X(96)-JVS(40)*X(104)&
            &-JVS(41)*X(120)-JVS(42)*X(143))/(JVS(33))
  X(16) = (X(16)-JVS(32)*X(131))/(JVS(31))
  X(15) = (X(15)-JVS(30)*X(129))/(JVS(29))
  X(14) = (X(14)-JVS(28)*X(142))/(JVS(27))
  X(13) = (X(13)-JVS(26)*X(140))/(JVS(25))
  X(12) = (X(12)-JVS(24)*X(132))/(JVS(23))
  X(11) = (X(11)-JVS(22)*X(39))/(JVS(21))
  X(10) = (X(10)-JVS(20)*X(42))/(JVS(19))
  X(9) = (X(9)-JVS(18)*X(83))/(JVS(17))
  X(8) = (X(8)-JVS(16)*X(44))/(JVS(15))
  X(7) = (X(7)-JVS(14)*X(137))/(JVS(13))
  X(6) = (X(6)-JVS(12)*X(141))/(JVS(11))
  X(5) = (X(5)-JVS(10)*X(43))/(JVS(9))
  X(4) = (X(4)-JVS(8)*X(98))/(JVS(7))
  X(3) = (X(3)-JVS(6)*X(66))/(JVS(5))
  X(2) = (X(2)-JVS(4)*X(125))/(JVS(3))
  X(1) = (X(1)-JVS(2)*X(88))/(JVS(1))
      
END SUBROUTINE KppSolve

! End of KppSolve function
! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~


! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
! 
! KppSolveTR - sparse, transposed back substitution
!   Arguments :
!      JVS       - sparse Jacobian of variables
!      X         - Vector for variables
!      XX        - Vector for output variables
! 
! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

SUBROUTINE KppSolveTR ( JVS, X, XX )

! JVS - sparse Jacobian of variables
  REAL(kind=dp) :: JVS(LU_NONZERO)
! X - Vector for variables
  REAL(kind=dp) :: X(NVAR)
! XX - Vector for output variables
  REAL(kind=dp) :: XX(NVAR)

  XX(1) = X(1)/JVS(1)
  XX(2) = X(2)/JVS(3)
  XX(3) = X(3)/JVS(5)
  XX(4) = X(4)/JVS(7)
  XX(5) = X(5)/JVS(9)
  XX(6) = X(6)/JVS(11)
  XX(7) = X(7)/JVS(13)
  XX(8) = X(8)/JVS(15)
  XX(9) = X(9)/JVS(17)
  XX(10) = X(10)/JVS(19)
  XX(11) = X(11)/JVS(21)
  XX(12) = X(12)/JVS(23)
  XX(13) = X(13)/JVS(25)
  XX(14) = X(14)/JVS(27)
  XX(15) = X(15)/JVS(29)
  XX(16) = X(16)/JVS(31)
  XX(17) = X(17)/JVS(33)
  XX(18) = X(18)/JVS(43)
  XX(19) = X(19)/JVS(46)
  XX(20) = X(20)/JVS(49)
  XX(21) = X(21)/JVS(51)
  XX(22) = X(22)/JVS(53)
  XX(23) = X(23)/JVS(55)
  XX(24) = X(24)/JVS(57)
  XX(25) = X(25)/JVS(60)
  XX(26) = X(26)/JVS(62)
  XX(27) = X(27)/JVS(64)
  XX(28) = X(28)/JVS(66)
  XX(29) = X(29)/JVS(68)
  XX(30) = X(30)/JVS(70)
  XX(31) = X(31)/JVS(72)
  XX(32) = X(32)/JVS(74)
  XX(33) = X(33)/JVS(75)
  XX(34) = X(34)/JVS(76)
  XX(35) = X(35)/JVS(96)
  XX(36) = X(36)/JVS(124)
  XX(37) = X(37)/JVS(126)
  XX(38) = X(38)/JVS(128)
  XX(39) = (X(39)-JVS(22)*XX(11)-JVS(97)*XX(35))/(JVS(130))
  XX(40) = X(40)/JVS(133)
  XX(41) = (X(41)-JVS(67)*XX(28)-JVS(98)*XX(35))/(JVS(136))
  XX(42) = (X(42)-JVS(20)*XX(10)-JVS(99)*XX(35))/(JVS(139))
  XX(43) = (X(43)-JVS(10)*XX(5)-JVS(100)*XX(35))/(JVS(142))
  XX(44) = (X(44)-JVS(16)*XX(8)-JVS(101)*XX(35))/(JVS(145))
  XX(45) = X(45)/JVS(148)
  XX(46) = X(46)/JVS(151)
  XX(47) = X(47)/JVS(155)
  XX(48) = X(48)/JVS(159)
  XX(49) = X(49)/JVS(162)
  XX(50) = X(50)/JVS(166)
  XX(51) = (X(51)-JVS(44)*XX(18))/(JVS(170))
  XX(52) = X(52)/JVS(173)
  XX(53) = X(53)/JVS(177)
  XX(54) = X(54)/JVS(180)
  XX(55) = X(55)/JVS(184)
  XX(56) = (X(56)-JVS(50)*XX(20)-JVS(102)*XX(35))/(JVS(188))
  XX(57) = (X(57)-JVS(56)*XX(23)-JVS(103)*XX(35)-JVS(189)*XX(56))/(JVS(195))
  XX(58) = (X(58)-JVS(52)*XX(21)-JVS(104)*XX(35)-JVS(196)*XX(57))/(JVS(203))
  XX(59) = X(59)/JVS(211)
  XX(60) = X(60)/JVS(215)
  XX(61) = X(61)/JVS(220)
  XX(62) = (X(62)-JVS(54)*XX(22)-JVS(105)*XX(35)-JVS(190)*XX(56)-JVS(197)*XX(57))/(JVS(224))
  XX(63) = (X(63)-JVS(77)*XX(34))/(JVS(228))
  XX(64) = (X(64)-JVS(204)*XX(58))/(JVS(235))
  XX(65) = X(65)/JVS(239)
  XX(66) = (X(66)-JVS(6)*XX(3)-JVS(106)*XX(35))/(JVS(243))
  XX(67) = X(67)/JVS(253)
  XX(68) = X(68)/JVS(259)
  XX(69) = X(69)/JVS(264)
  XX(70) = (X(70)-JVS(65)*XX(27)-JVS(107)*XX(35)-JVS(137)*XX(41))/(JVS(268))
  XX(71) = (X(71)-JVS(78)*XX(34))/(JVS(272))
  XX(72) = X(72)/JVS(277)
  XX(73) = X(73)/JVS(282)
  XX(74) = (X(74)-JVS(79)*XX(34))/(JVS(288))
  XX(75) = X(75)/JVS(294)
  XX(76) = X(76)/JVS(300)
  XX(77) = (X(77)-JVS(63)*XX(26)-JVS(108)*XX(35))/(JVS(306))
  XX(78) = (X(78)-JVS(80)*XX(34))/(JVS(313))
  XX(79) = (X(79)-JVS(81)*XX(34)-JVS(140)*XX(42))/(JVS(319))
  XX(80) = (X(80)-JVS(82)*XX(34)-JVS(254)*XX(67))/(JVS(326))
  XX(81) = X(81)/JVS(333)
  XX(82) = (X(82)-JVS(255)*XX(67))/(JVS(348))
  XX(83) = (X(83)-JVS(18)*XX(9)-JVS(109)*XX(35))/(JVS(353))
  XX(84) = (X(84)-JVS(34)*XX(17)-JVS(244)*XX(66))/(JVS(359))
  XX(85) = (X(85)-JVS(35)*XX(17)-JVS(245)*XX(66))/(JVS(366))
  XX(86) = (X(86)-JVS(58)*XX(24)-JVS(71)*XX(30)-JVS(110)*XX(35))/(JVS(374))
  XX(87) = (X(87)-JVS(36)*XX(17)-JVS(246)*XX(66))/(JVS(382))
  XX(88) = (X(88)-JVS(2)*XX(1)-JVS(111)*XX(35)-JVS(205)*XX(58))/(JVS(390))
  XX(89) = (X(89)-JVS(295)*XX(75))/(JVS(400))
  XX(90) = (X(90)-JVS(37)*XX(17)-JVS(247)*XX(66))/(JVS(406))
  XX(91) = (X(91)-JVS(47)*XX(19)-JVS(360)*XX(84)-JVS(407)*XX(90))/(JVS(415))
  XX(92) = (X(92)-JVS(367)*XX(85)-JVS(408)*XX(90))/(JVS(420))
  XX(93) = (X(93)-JVS(191)*XX(56)-JVS(198)*XX(57)-JVS(225)*XX(62))/(JVS(426))
  XX(94) = (X(94)-JVS(38)*XX(17)-JVS(248)*XX(66))/(JVS(434))
  XX(95) = (X(95)-JVS(73)*XX(31)-JVS(112)*XX(35))/(JVS(443))
  XX(96) = (X(96)-JVS(39)*XX(17)-JVS(249)*XX(66))/(JVS(454))
  XX(97) = (X(97)-JVS(296)*XX(75))/(JVS(461))
  XX(98) = (X(98)-JVS(8)*XX(4)-JVS(113)*XX(35))/(JVS(475))
  XX(99) = (X(99)-JVS(163)*XX(49)-JVS(334)*XX(81))/(JVS(499))
  XX(100) = (X(100)-JVS(476)*XX(98))/(JVS(512))
  XX(101) = (X(101)-JVS(83)*XX(34)-JVS(335)*XX(81)-JVS(477)*XX(98))/(JVS(525))
  XX(102) = (X(102)-JVS(61)*XX(25)-JVS(114)*XX(35)-JVS(229)*XX(63)-JVS(444)*XX(95))/(JVS(532))
  XX(103) = (X(103)-JVS(69)*XX(29)-JVS(115)*XX(35)-JVS(320)*XX(79))/(JVS(540))
  XX(104) = (X(104)-JVS(40)*XX(17)-JVS(250)*XX(66)-JVS(273)*XX(71))/(JVS(550))
  XX(105) = (X(105)-JVS(240)*XX(65)-JVS(336)*XX(81))/(JVS(560))
  XX(106) = (X(106)-JVS(216)*XX(60)-JVS(354)*XX(83))/(JVS(569))
  XX(107) = (X(107)-JVS(84)*XX(34)-JVS(301)*XX(76)-JVS(445)*XX(95))/(JVS(576))
  XX(108) = (X(108)-JVS(167)*XX(50)-JVS(337)*XX(81)-JVS(391)*XX(88))/(JVS(585))
  XX(109) = (X(109)-JVS(221)*XX(61)-JVS(338)*XX(81)-JVS(446)*XX(95))/(JVS(596))
  XX(110) = (X(110)-JVS(260)*XX(68)-JVS(339)*XX(81))/(JVS(605))
  XX(111) = (X(111)-JVS(265)*XX(69)-JVS(340)*XX(81)-JVS(375)*XX(86))/(JVS(615))
  XX(112) = (X(112)-JVS(278)*XX(72)-JVS(283)*XX(73)-JVS(289)*XX(74)-JVS(327)*XX(80)-JVS(341)*XX(81)-JVS(478)*XX(98)&
              &-JVS(513)*XX(100))/(JVS(625))
  XX(113) = (X(113)-JVS(181)*XX(54))/(JVS(634))
  XX(114) = (X(114)-JVS(85)*XX(34)-JVS(514)*XX(100))/(JVS(656))
  XX(115) = (X(115)-JVS(131)*XX(39)-JVS(217)*XX(60)-JVS(635)*XX(113))/(JVS(676))
  XX(116) = (X(116)-JVS(261)*XX(68))/(JVS(692))
  XX(117) = (X(117)-JVS(342)*XX(81))/(JVS(705))
  XX(118) = (X(118)-JVS(401)*XX(89)-JVS(462)*XX(97)-JVS(479)*XX(98)-JVS(515)*XX(100)-JVS(570)*XX(106)-JVS(677)*XX(115))&
              &/(JVS(716))
  XX(119) = (X(119)-JVS(86)*XX(34)-JVS(212)*XX(59)-JVS(355)*XX(83)-JVS(526)*XX(101))/(JVS(728))
  XX(120) = (X(120)-JVS(41)*XX(17)-JVS(361)*XX(84)-JVS(368)*XX(85)-JVS(383)*XX(87)-JVS(409)*XX(90)-JVS(435)*XX(94)&
              &-JVS(455)*XX(96)-JVS(551)*XX(104)-JVS(657)*XX(114))/(JVS(749))
  XX(121) = (X(121)-JVS(480)*XX(98))/(JVS(778))
  XX(122) = (X(122)-JVS(87)*XX(34)-JVS(269)*XX(70)-JVS(307)*XX(77)-JVS(314)*XX(78)-JVS(343)*XX(81)-JVS(349)*XX(82)&
              &-JVS(376)*XX(86)-JVS(481)*XX(98)-JVS(516)*XX(100)-JVS(533)*XX(102)-JVS(541)*XX(103)-JVS(658)*XX(114)-JVS(779)&
              &*XX(121))/(JVS(803))
  XX(123) = (X(123)-JVS(297)*XX(75)-JVS(534)*XX(102)-JVS(659)*XX(114)-JVS(717)*XX(118)-JVS(729)*XX(119)-JVS(780)&
              &*XX(121))/(JVS(816))
  XX(124) = (X(124)-JVS(384)*XX(87)-JVS(436)*XX(94)-JVS(626)*XX(112)-JVS(660)*XX(114)-JVS(750)*XX(120)-JVS(781)*XX(121))&
              &/(JVS(829))
  XX(125) = (X(125)-JVS(4)*XX(2)-JVS(116)*XX(35)-JVS(206)*XX(58)-JVS(482)*XX(98))/(JVS(883))
  XX(126) = (X(126)-JVS(274)*XX(71)-JVS(344)*XX(81)-JVS(542)*XX(103)-JVS(782)*XX(121)-JVS(884)*XX(125))/(JVS(909))
  XX(127) = (X(127)-JVS(302)*XX(76)-JVS(377)*XX(86)-JVS(483)*XX(98)-JVS(543)*XX(103)-JVS(661)*XX(114)-JVS(706)*XX(117)&
              &-JVS(783)*XX(121)-JVS(885)*XX(125))/(JVS(925))
  XX(128) = (X(128)-JVS(230)*XX(63)-JVS(303)*XX(76)-JVS(447)*XX(95)-JVS(484)*XX(98)-JVS(544)*XX(103)-JVS(662)*XX(114)&
              &-JVS(707)*XX(117)-JVS(784)*XX(121)-JVS(886)*XX(125)-JVS(926)*XX(127))/(JVS(941))
  XX(129) = (X(129)-JVS(30)*XX(15)-JVS(88)*XX(34)-JVS(117)*XX(35)-JVS(785)*XX(121)-JVS(887)*XX(125))/(JVS(976))
  XX(130) = (X(130)-JVS(256)*XX(67)-JVS(345)*XX(81)-JVS(392)*XX(88)-JVS(500)*XX(99)-JVS(586)*XX(108)-JVS(636)*XX(113)&
              &-JVS(693)*XX(116)-JVS(786)*XX(121)-JVS(888)*XX(125)-JVS(977)*XX(129))/(JVS(1002))
  XX(131) = (X(131)-JVS(32)*XX(16)-JVS(118)*XX(35)-JVS(207)*XX(58)-JVS(485)*XX(98)-JVS(889)*XX(125))/(JVS(1031))
  XX(132) = (X(132)-JVS(24)*XX(12)-JVS(119)*XX(35)-JVS(393)*XX(88)-JVS(501)*XX(99)-JVS(587)*XX(108)-JVS(637)*XX(113)&
              &-JVS(694)*XX(116)-JVS(708)*XX(117)-JVS(787)*XX(121)-JVS(890)*XX(125)-JVS(978)*XX(129)-JVS(1003)*XX(130)&
              &-JVS(1032)*XX(131))/(JVS(1060))
  XX(133) = (X(133)-JVS(486)*XX(98)-JVS(638)*XX(113)-JVS(678)*XX(115)-JVS(891)*XX(125)-JVS(1033)*XX(131))/(JVS(1102))
  XX(134) = (X(134)-JVS(134)*XX(40)-JVS(178)*XX(53)-JVS(199)*XX(57)-JVS(208)*XX(58)-JVS(226)*XX(62)-JVS(236)*XX(64)&
              &-JVS(394)*XX(88)-JVS(427)*XX(93)-JVS(487)*XX(98)-JVS(502)*XX(99)-JVS(588)*XX(108)-JVS(639)*XX(113)-JVS(695)&
              &*XX(116)-JVS(788)*XX(121)-JVS(892)*XX(125)-JVS(979)*XX(129)-JVS(1034)*XX(131)-JVS(1061)*XX(132)-JVS(1103)&
              &*XX(133))/(JVS(1138))
  XX(135) = (X(135)-JVS(488)*XX(98)-JVS(606)*XX(110)-JVS(640)*XX(113)-JVS(679)*XX(115)-JVS(696)*XX(116)-JVS(789)*XX(121)&
              &-JVS(893)*XX(125)-JVS(980)*XX(129)-JVS(1035)*XX(131)-JVS(1062)*XX(132)-JVS(1104)*XX(133)-JVS(1139)*XX(134))&
              &/(JVS(1172))
  XX(136) = (X(136)-JVS(89)*XX(34)-JVS(362)*XX(84)-JVS(369)*XX(85)-JVS(385)*XX(87)-JVS(437)*XX(94)-JVS(456)*XX(96)&
              &-JVS(552)*XX(104)-JVS(663)*XX(114)-JVS(751)*XX(120)-JVS(790)*XX(121)-JVS(894)*XX(125)-JVS(910)*XX(126)&
              &-JVS(981)*XX(129)-JVS(1036)*XX(131)-JVS(1105)*XX(133)-JVS(1140)*XX(134)-JVS(1173)*XX(135))/(JVS(1228))
  XX(137) = (X(137)-JVS(14)*XX(7)-JVS(90)*XX(34)-JVS(120)*XX(35)-JVS(231)*XX(63)-JVS(290)*XX(74)-JVS(308)*XX(77)&
              &-JVS(328)*XX(80)-JVS(356)*XX(83)-JVS(363)*XX(84)-JVS(370)*XX(85)-JVS(378)*XX(86)-JVS(386)*XX(87)-JVS(410)&
              &*XX(90)-JVS(416)*XX(91)-JVS(421)*XX(92)-JVS(428)*XX(93)-JVS(438)*XX(94)-JVS(448)*XX(95)-JVS(457)*XX(96)&
              &-JVS(489)*XX(98)-JVS(517)*XX(100)-JVS(535)*XX(102)-JVS(545)*XX(103)-JVS(553)*XX(104)-JVS(561)*XX(105)&
              &-JVS(597)*XX(109)-JVS(616)*XX(111)-JVS(664)*XX(114)-JVS(718)*XX(118)-JVS(730)*XX(119)-JVS(752)*XX(120)&
              &-JVS(791)*XX(121)-JVS(804)*XX(122)-JVS(817)*XX(123)-JVS(830)*XX(124)-JVS(895)*XX(125)-JVS(911)*XX(126)&
              &-JVS(927)*XX(127)-JVS(942)*XX(128)-JVS(982)*XX(129)-JVS(1004)*XX(130)-JVS(1037)*XX(131)-JVS(1063)*XX(132)&
              &-JVS(1106)*XX(133)-JVS(1141)*XX(134)-JVS(1174)*XX(135)-JVS(1229)*XX(136))/(JVS(1258))
  XX(138) = (X(138)-JVS(91)*XX(34)-JVS(156)*XX(47)-JVS(232)*XX(63)-JVS(284)*XX(73)-JVS(291)*XX(74)-JVS(309)*XX(77)&
              &-JVS(315)*XX(78)-JVS(321)*XX(79)-JVS(329)*XX(80)-JVS(350)*XX(82)-JVS(364)*XX(84)-JVS(371)*XX(85)-JVS(379)&
              &*XX(86)-JVS(387)*XX(87)-JVS(395)*XX(88)-JVS(402)*XX(89)-JVS(411)*XX(90)-JVS(429)*XX(93)-JVS(439)*XX(94)&
              &-JVS(449)*XX(95)-JVS(458)*XX(96)-JVS(463)*XX(97)-JVS(490)*XX(98)-JVS(503)*XX(99)-JVS(518)*XX(100)-JVS(527)&
              &*XX(101)-JVS(536)*XX(102)-JVS(546)*XX(103)-JVS(554)*XX(104)-JVS(562)*XX(105)-JVS(571)*XX(106)-JVS(577)&
              &*XX(107)-JVS(589)*XX(108)-JVS(598)*XX(109)-JVS(607)*XX(110)-JVS(617)*XX(111)-JVS(627)*XX(112)-JVS(641)&
              &*XX(113)-JVS(665)*XX(114)-JVS(680)*XX(115)-JVS(697)*XX(116)-JVS(709)*XX(117)-JVS(719)*XX(118)-JVS(731)&
              &*XX(119)-JVS(753)*XX(120)-JVS(792)*XX(121)-JVS(805)*XX(122)-JVS(818)*XX(123)-JVS(831)*XX(124)-JVS(896)&
              &*XX(125)-JVS(912)*XX(126)-JVS(928)*XX(127)-JVS(943)*XX(128)-JVS(983)*XX(129)-JVS(1005)*XX(130)-JVS(1038)&
              &*XX(131)-JVS(1064)*XX(132)-JVS(1107)*XX(133)-JVS(1142)*XX(134)-JVS(1175)*XX(135)-JVS(1230)*XX(136)-JVS(1259)&
              &*XX(137))/(JVS(1316))
  XX(139) = (X(139)-JVS(92)*XX(34)-JVS(146)*XX(44)-JVS(152)*XX(46)-JVS(285)*XX(73)-JVS(292)*XX(74)-JVS(330)*XX(80)&
              &-JVS(351)*XX(82)-JVS(396)*XX(88)-JVS(450)*XX(95)-JVS(491)*XX(98)-JVS(504)*XX(99)-JVS(519)*XX(100)-JVS(528)&
              &*XX(101)-JVS(563)*XX(105)-JVS(578)*XX(107)-JVS(590)*XX(108)-JVS(599)*XX(109)-JVS(608)*XX(110)-JVS(618)&
              &*XX(111)-JVS(628)*XX(112)-JVS(642)*XX(113)-JVS(666)*XX(114)-JVS(681)*XX(115)-JVS(698)*XX(116)-JVS(710)&
              &*XX(117)-JVS(720)*XX(118)-JVS(732)*XX(119)-JVS(793)*XX(121)-JVS(806)*XX(122)-JVS(819)*XX(123)-JVS(832)&
              &*XX(124)-JVS(897)*XX(125)-JVS(913)*XX(126)-JVS(929)*XX(127)-JVS(944)*XX(128)-JVS(984)*XX(129)-JVS(1006)&
              &*XX(130)-JVS(1039)*XX(131)-JVS(1065)*XX(132)-JVS(1108)*XX(133)-JVS(1143)*XX(134)-JVS(1176)*XX(135)-JVS(1231)&
              &*XX(136)-JVS(1260)*XX(137)-JVS(1317)*XX(138))/(JVS(1369))
  XX(140) = (X(140)-JVS(26)*XX(13)-JVS(121)*XX(35)-JVS(412)*XX(90)-JVS(422)*XX(92)-JVS(459)*XX(96)-JVS(564)*XX(105)&
              &-JVS(600)*XX(109)-JVS(619)*XX(111)-JVS(667)*XX(114)-JVS(682)*XX(115)-JVS(754)*XX(120)-JVS(794)*XX(121)&
              &-JVS(898)*XX(125)-JVS(914)*XX(126)-JVS(985)*XX(129)-JVS(1040)*XX(131)-JVS(1109)*XX(133)-JVS(1144)*XX(134)&
              &-JVS(1177)*XX(135)-JVS(1232)*XX(136)-JVS(1261)*XX(137)-JVS(1318)*XX(138)-JVS(1370)*XX(139))/(JVS(1388))
  XX(141) = (X(141)-JVS(12)*XX(6)-JVS(122)*XX(35)-JVS(132)*XX(39)-JVS(135)*XX(40)-JVS(141)*XX(42)-JVS(143)*XX(43)&
              &-JVS(147)*XX(44)-JVS(157)*XX(47)-JVS(160)*XX(48)-JVS(185)*XX(55)-JVS(227)*XX(62)-JVS(322)*XX(79)-JVS(357)&
              &*XX(83)-JVS(365)*XX(84)-JVS(372)*XX(85)-JVS(388)*XX(87)-JVS(413)*XX(90)-JVS(430)*XX(93)-JVS(440)*XX(94)&
              &-JVS(460)*XX(96)-JVS(492)*XX(98)-JVS(555)*XX(104)-JVS(572)*XX(106)-JVS(668)*XX(114)-JVS(683)*XX(115)-JVS(733)&
              &*XX(119)-JVS(755)*XX(120)-JVS(795)*XX(121)-JVS(899)*XX(125)-JVS(915)*XX(126)-JVS(986)*XX(129)-JVS(1041)&
              &*XX(131)-JVS(1066)*XX(132)-JVS(1110)*XX(133)-JVS(1145)*XX(134)-JVS(1178)*XX(135)-JVS(1233)*XX(136)-JVS(1262)&
              &*XX(137)-JVS(1319)*XX(138)-JVS(1371)*XX(139)-JVS(1389)*XX(140))/(JVS(1470))
  XX(142) = (X(142)-JVS(28)*XX(14)-JVS(123)*XX(35)-JVS(414)*XX(90)-JVS(441)*XX(94)-JVS(493)*XX(98)-JVS(579)*XX(107)&
              &-JVS(711)*XX(117)-JVS(734)*XX(119)-JVS(756)*XX(120)-JVS(796)*XX(121)-JVS(900)*XX(125)-JVS(916)*XX(126)&
              &-JVS(930)*XX(127)-JVS(945)*XX(128)-JVS(987)*XX(129)-JVS(1007)*XX(130)-JVS(1042)*XX(131)-JVS(1067)*XX(132)&
              &-JVS(1111)*XX(133)-JVS(1146)*XX(134)-JVS(1179)*XX(135)-JVS(1234)*XX(136)-JVS(1263)*XX(137)-JVS(1320)*XX(138)&
              &-JVS(1372)*XX(139)-JVS(1390)*XX(140)-JVS(1471)*XX(141))/(JVS(1489))
  XX(143) = (X(143)-JVS(42)*XX(17)-JVS(45)*XX(18)-JVS(48)*XX(19)-JVS(59)*XX(24)-JVS(93)*XX(34)-JVS(125)*XX(36)-JVS(127)&
              &*XX(37)-JVS(129)*XX(38)-JVS(138)*XX(41)-JVS(149)*XX(45)-JVS(153)*XX(46)-JVS(158)*XX(47)-JVS(164)*XX(49)&
              &-JVS(168)*XX(50)-JVS(171)*XX(51)-JVS(174)*XX(52)-JVS(179)*XX(53)-JVS(182)*XX(54)-JVS(186)*XX(55)-JVS(192)&
              &*XX(56)-JVS(200)*XX(57)-JVS(209)*XX(58)-JVS(213)*XX(59)-JVS(218)*XX(60)-JVS(222)*XX(61)-JVS(233)*XX(63)&
              &-JVS(237)*XX(64)-JVS(241)*XX(65)-JVS(251)*XX(66)-JVS(257)*XX(67)-JVS(262)*XX(68)-JVS(266)*XX(69)-JVS(270)&
              &*XX(70)-JVS(275)*XX(71)-JVS(279)*XX(72)-JVS(286)*XX(73)-JVS(293)*XX(74)-JVS(298)*XX(75)-JVS(304)*XX(76)&
              &-JVS(310)*XX(77)-JVS(316)*XX(78)-JVS(323)*XX(79)-JVS(331)*XX(80)-JVS(346)*XX(81)-JVS(358)*XX(83)-JVS(380)&
              &*XX(86)-JVS(397)*XX(88)-JVS(403)*XX(89)-JVS(417)*XX(91)-JVS(423)*XX(92)-JVS(431)*XX(93)-JVS(451)*XX(95)&
              &-JVS(494)*XX(98)-JVS(505)*XX(99)-JVS(520)*XX(100)-JVS(529)*XX(101)-JVS(537)*XX(102)-JVS(547)*XX(103)-JVS(556)&
              &*XX(104)-JVS(565)*XX(105)-JVS(573)*XX(106)-JVS(591)*XX(108)-JVS(601)*XX(109)-JVS(609)*XX(110)-JVS(620)&
              &*XX(111)-JVS(629)*XX(112)-JVS(643)*XX(113)-JVS(669)*XX(114)-JVS(684)*XX(115)-JVS(699)*XX(116)-JVS(712)&
              &*XX(117)-JVS(721)*XX(118)-JVS(735)*XX(119)-JVS(757)*XX(120)-JVS(797)*XX(121)-JVS(807)*XX(122)-JVS(820)&
              &*XX(123)-JVS(833)*XX(124)-JVS(901)*XX(125)-JVS(917)*XX(126)-JVS(931)*XX(127)-JVS(946)*XX(128)-JVS(988)&
              &*XX(129)-JVS(1008)*XX(130)-JVS(1043)*XX(131)-JVS(1068)*XX(132)-JVS(1112)*XX(133)-JVS(1147)*XX(134)-JVS(1180)&
              &*XX(135)-JVS(1235)*XX(136)-JVS(1264)*XX(137)-JVS(1321)*XX(138)-JVS(1373)*XX(139)-JVS(1391)*XX(140)-JVS(1472)&
              &*XX(141)-JVS(1490)*XX(142))/(JVS(1595))
  XX(144) = (X(144)-JVS(94)*XX(34)-JVS(154)*XX(46)-JVS(165)*XX(49)-JVS(169)*XX(50)-JVS(175)*XX(52)-JVS(183)*XX(54)&
              &-JVS(187)*XX(55)-JVS(193)*XX(56)-JVS(201)*XX(57)-JVS(210)*XX(58)-JVS(214)*XX(59)-JVS(219)*XX(60)-JVS(223)&
              &*XX(61)-JVS(234)*XX(63)-JVS(242)*XX(65)-JVS(252)*XX(66)-JVS(258)*XX(67)-JVS(263)*XX(68)-JVS(267)*XX(69)&
              &-JVS(271)*XX(70)-JVS(276)*XX(71)-JVS(280)*XX(72)-JVS(287)*XX(73)-JVS(299)*XX(75)-JVS(305)*XX(76)-JVS(317)&
              &*XX(78)-JVS(324)*XX(79)-JVS(332)*XX(80)-JVS(352)*XX(82)-JVS(381)*XX(86)-JVS(398)*XX(88)-JVS(404)*XX(89)&
              &-JVS(432)*XX(93)-JVS(452)*XX(95)-JVS(464)*XX(97)-JVS(495)*XX(98)-JVS(506)*XX(99)-JVS(521)*XX(100)-JVS(530)&
              &*XX(101)-JVS(548)*XX(103)-JVS(566)*XX(105)-JVS(574)*XX(106)-JVS(580)*XX(107)-JVS(592)*XX(108)-JVS(602)&
              &*XX(109)-JVS(610)*XX(110)-JVS(621)*XX(111)-JVS(630)*XX(112)-JVS(644)*XX(113)-JVS(670)*XX(114)-JVS(685)&
              &*XX(115)-JVS(700)*XX(116)-JVS(713)*XX(117)-JVS(736)*XX(119)-JVS(798)*XX(121)-JVS(808)*XX(122)-JVS(821)&
              &*XX(123)-JVS(834)*XX(124)-JVS(902)*XX(125)-JVS(918)*XX(126)-JVS(932)*XX(127)-JVS(947)*XX(128)-JVS(989)&
              &*XX(129)-JVS(1009)*XX(130)-JVS(1044)*XX(131)-JVS(1069)*XX(132)-JVS(1113)*XX(133)-JVS(1148)*XX(134)-JVS(1181)&
              &*XX(135)-JVS(1236)*XX(136)-JVS(1265)*XX(137)-JVS(1322)*XX(138)-JVS(1374)*XX(139)-JVS(1392)*XX(140)-JVS(1473)&
              &*XX(141)-JVS(1491)*XX(142)-JVS(1596)*XX(143))/(JVS(1680))
  XX(145) = (X(145)-JVS(95)*XX(34)-JVS(161)*XX(48)-JVS(176)*XX(52)-JVS(347)*XX(81)-JVS(399)*XX(88)-JVS(405)*XX(89)&
              &-JVS(453)*XX(95)-JVS(465)*XX(97)-JVS(507)*XX(99)-JVS(522)*XX(100)-JVS(531)*XX(101)-JVS(538)*XX(102)-JVS(567)&
              &*XX(105)-JVS(581)*XX(107)-JVS(593)*XX(108)-JVS(603)*XX(109)-JVS(611)*XX(110)-JVS(622)*XX(111)-JVS(631)&
              &*XX(112)-JVS(645)*XX(113)-JVS(671)*XX(114)-JVS(686)*XX(115)-JVS(701)*XX(116)-JVS(714)*XX(117)-JVS(722)&
              &*XX(118)-JVS(737)*XX(119)-JVS(799)*XX(121)-JVS(809)*XX(122)-JVS(822)*XX(123)-JVS(835)*XX(124)-JVS(903)&
              &*XX(125)-JVS(919)*XX(126)-JVS(933)*XX(127)-JVS(948)*XX(128)-JVS(990)*XX(129)-JVS(1010)*XX(130)-JVS(1045)&
              &*XX(131)-JVS(1070)*XX(132)-JVS(1114)*XX(133)-JVS(1149)*XX(134)-JVS(1182)*XX(135)-JVS(1237)*XX(136)-JVS(1266)&
              &*XX(137)-JVS(1323)*XX(138)-JVS(1375)*XX(139)-JVS(1393)*XX(140)-JVS(1474)*XX(141)-JVS(1492)*XX(142)-JVS(1597)&
              &*XX(143)-JVS(1681)*XX(144))/(JVS(1734))
  XX(146) = (X(146)-JVS(144)*XX(43)-JVS(150)*XX(45)-JVS(172)*XX(51)-JVS(238)*XX(64)-JVS(418)*XX(91)-JVS(424)*XX(92)&
              &-JVS(433)*XX(93)-JVS(466)*XX(97)-JVS(496)*XX(98)-JVS(523)*XX(100)-JVS(557)*XX(104)-JVS(568)*XX(105)-JVS(575)&
              &*XX(106)-JVS(582)*XX(107)-JVS(604)*XX(109)-JVS(612)*XX(110)-JVS(623)*XX(111)-JVS(646)*XX(113)-JVS(672)&
              &*XX(114)-JVS(687)*XX(115)-JVS(702)*XX(116)-JVS(715)*XX(117)-JVS(723)*XX(118)-JVS(738)*XX(119)-JVS(758)&
              &*XX(120)-JVS(800)*XX(121)-JVS(810)*XX(122)-JVS(823)*XX(123)-JVS(836)*XX(124)-JVS(904)*XX(125)-JVS(920)&
              &*XX(126)-JVS(934)*XX(127)-JVS(949)*XX(128)-JVS(991)*XX(129)-JVS(1011)*XX(130)-JVS(1046)*XX(131)-JVS(1071)&
              &*XX(132)-JVS(1115)*XX(133)-JVS(1150)*XX(134)-JVS(1183)*XX(135)-JVS(1238)*XX(136)-JVS(1267)*XX(137)-JVS(1324)&
              &*XX(138)-JVS(1376)*XX(139)-JVS(1394)*XX(140)-JVS(1475)*XX(141)-JVS(1493)*XX(142)-JVS(1598)*XX(143)-JVS(1682)&
              &*XX(144)-JVS(1735)*XX(145))/(JVS(1794))
  XX(146) = XX(146)
  XX(145) = XX(145)-JVS(1793)*XX(146)
  XX(144) = XX(144)-JVS(1733)*XX(145)-JVS(1792)*XX(146)
  XX(143) = XX(143)-JVS(1679)*XX(144)-JVS(1732)*XX(145)-JVS(1791)*XX(146)
  XX(142) = XX(142)-JVS(1594)*XX(143)-JVS(1678)*XX(144)-JVS(1731)*XX(145)-JVS(1790)*XX(146)
  XX(141) = XX(141)-JVS(1488)*XX(142)-JVS(1593)*XX(143)-JVS(1677)*XX(144)-JVS(1730)*XX(145)-JVS(1789)*XX(146)
  XX(140) = XX(140)-JVS(1469)*XX(141)-JVS(1487)*XX(142)-JVS(1592)*XX(143)-JVS(1676)*XX(144)-JVS(1729)*XX(145)-JVS(1788)&
              &*XX(146)
  XX(139) = XX(139)-JVS(1387)*XX(140)-JVS(1468)*XX(141)-JVS(1486)*XX(142)-JVS(1591)*XX(143)-JVS(1675)*XX(144)-JVS(1728)&
              &*XX(145)-JVS(1787)*XX(146)
  XX(138) = XX(138)-JVS(1368)*XX(139)-JVS(1386)*XX(140)-JVS(1467)*XX(141)-JVS(1485)*XX(142)-JVS(1590)*XX(143)-JVS(1674)&
              &*XX(144)-JVS(1727)*XX(145)-JVS(1786)*XX(146)
  XX(137) = XX(137)-JVS(1315)*XX(138)-JVS(1367)*XX(139)-JVS(1385)*XX(140)-JVS(1466)*XX(141)-JVS(1484)*XX(142)-JVS(1589)&
              &*XX(143)-JVS(1673)*XX(144)-JVS(1726)*XX(145)-JVS(1785)*XX(146)
  XX(136) = XX(136)-JVS(1257)*XX(137)-JVS(1314)*XX(138)-JVS(1366)*XX(139)-JVS(1384)*XX(140)-JVS(1465)*XX(141)-JVS(1483)&
              &*XX(142)-JVS(1588)*XX(143)-JVS(1672)*XX(144)-JVS(1725)*XX(145)-JVS(1784)*XX(146)
  XX(135) = XX(135)-JVS(1227)*XX(136)-JVS(1256)*XX(137)-JVS(1313)*XX(138)-JVS(1365)*XX(139)-JVS(1464)*XX(141)-JVS(1587)&
              &*XX(143)-JVS(1671)*XX(144)-JVS(1724)*XX(145)-JVS(1783)*XX(146)
  XX(134) = XX(134)-JVS(1171)*XX(135)-JVS(1226)*XX(136)-JVS(1255)*XX(137)-JVS(1312)*XX(138)-JVS(1364)*XX(139)-JVS(1463)&
              &*XX(141)-JVS(1586)*XX(143)-JVS(1670)*XX(144)-JVS(1723)*XX(145)-JVS(1782)*XX(146)
  XX(133) = XX(133)-JVS(1137)*XX(134)-JVS(1225)*XX(136)-JVS(1254)*XX(137)-JVS(1311)*XX(138)-JVS(1363)*XX(139)-JVS(1462)&
              &*XX(141)-JVS(1585)*XX(143)-JVS(1669)*XX(144)-JVS(1722)*XX(145)-JVS(1781)*XX(146)
  XX(132) = XX(132)-JVS(1101)*XX(133)-JVS(1136)*XX(134)-JVS(1170)*XX(135)-JVS(1224)*XX(136)-JVS(1310)*XX(138)-JVS(1362)&
              &*XX(139)-JVS(1461)*XX(141)-JVS(1584)*XX(143)-JVS(1668)*XX(144)-JVS(1721)*XX(145)-JVS(1780)*XX(146)
  XX(131) = XX(131)-JVS(1135)*XX(134)-JVS(1223)*XX(136)-JVS(1361)*XX(139)-JVS(1460)*XX(141)-JVS(1583)*XX(143)-JVS(1667)&
              &*XX(144)-JVS(1720)*XX(145)-JVS(1779)*XX(146)
  XX(130) = XX(130)-JVS(1030)*XX(131)-JVS(1059)*XX(132)-JVS(1100)*XX(133)-JVS(1134)*XX(134)-JVS(1169)*XX(135)-JVS(1222)&
              &*XX(136)-JVS(1309)*XX(138)-JVS(1360)*XX(139)-JVS(1459)*XX(141)-JVS(1582)*XX(143)-JVS(1666)*XX(144)-JVS(1719)&
              &*XX(145)-JVS(1778)*XX(146)
  XX(129) = XX(129)-JVS(1133)*XX(134)-JVS(1221)*XX(136)-JVS(1359)*XX(139)-JVS(1458)*XX(141)-JVS(1581)*XX(143)-JVS(1665)&
              &*XX(144)-JVS(1718)*XX(145)-JVS(1777)*XX(146)
  XX(128) = XX(128)-JVS(975)*XX(129)-JVS(1001)*XX(130)-JVS(1029)*XX(131)-JVS(1058)*XX(132)-JVS(1099)*XX(133)-JVS(1132)&
              &*XX(134)-JVS(1168)*XX(135)-JVS(1220)*XX(136)-JVS(1308)*XX(138)-JVS(1358)*XX(139)-JVS(1457)*XX(141)-JVS(1580)&
              &*XX(143)-JVS(1664)*XX(144)-JVS(1717)*XX(145)-JVS(1776)*XX(146)
  XX(127) = XX(127)-JVS(940)*XX(128)-JVS(974)*XX(129)-JVS(1000)*XX(130)-JVS(1028)*XX(131)-JVS(1057)*XX(132)-JVS(1098)&
              &*XX(133)-JVS(1131)*XX(134)-JVS(1219)*XX(136)-JVS(1307)*XX(138)-JVS(1357)*XX(139)-JVS(1456)*XX(141)-JVS(1579)&
              &*XX(143)-JVS(1663)*XX(144)-JVS(1716)*XX(145)-JVS(1775)*XX(146)
  XX(126) = XX(126)-JVS(973)*XX(129)-JVS(1097)*XX(133)-JVS(1130)*XX(134)-JVS(1167)*XX(135)-JVS(1218)*XX(136)-JVS(1306)&
              &*XX(138)-JVS(1356)*XX(139)-JVS(1455)*XX(141)-JVS(1578)*XX(143)-JVS(1662)*XX(144)-JVS(1715)*XX(145)-JVS(1774)&
              &*XX(146)
  XX(125) = XX(125)-JVS(1129)*XX(134)-JVS(1217)*XX(136)-JVS(1454)*XX(141)-JVS(1577)*XX(143)-JVS(1661)*XX(144)-JVS(1773)&
              &*XX(146)
  XX(124) = XX(124)-JVS(882)*XX(125)-JVS(908)*XX(126)-JVS(972)*XX(129)-JVS(999)*XX(130)-JVS(1027)*XX(131)-JVS(1096)&
              &*XX(133)-JVS(1166)*XX(135)-JVS(1216)*XX(136)-JVS(1253)*XX(137)-JVS(1305)*XX(138)-JVS(1355)*XX(139)-JVS(1383)&
              &*XX(140)-JVS(1453)*XX(141)-JVS(1482)*XX(142)-JVS(1576)*XX(143)-JVS(1660)*XX(144)-JVS(1714)*XX(145)-JVS(1772)&
              &*XX(146)
  XX(123) = XX(123)-JVS(828)*XX(124)-JVS(881)*XX(125)-JVS(939)*XX(128)-JVS(971)*XX(129)-JVS(1026)*XX(131)-JVS(1056)&
              &*XX(132)-JVS(1095)*XX(133)-JVS(1165)*XX(135)-JVS(1215)*XX(136)-JVS(1252)*XX(137)-JVS(1304)*XX(138)-JVS(1354)&
              &*XX(139)-JVS(1382)*XX(140)-JVS(1452)*XX(141)-JVS(1481)*XX(142)-JVS(1575)*XX(143)-JVS(1659)*XX(144)-JVS(1713)&
              &*XX(145)-JVS(1771)*XX(146)
  XX(122) = XX(122)-JVS(827)*XX(124)-JVS(880)*XX(125)-JVS(924)*XX(127)-JVS(938)*XX(128)-JVS(970)*XX(129)-JVS(998)&
              &*XX(130)-JVS(1094)*XX(133)-JVS(1164)*XX(135)-JVS(1214)*XX(136)-JVS(1251)*XX(137)-JVS(1303)*XX(138)-JVS(1353)&
              &*XX(139)-JVS(1381)*XX(140)-JVS(1451)*XX(141)-JVS(1480)*XX(142)-JVS(1574)*XX(143)-JVS(1658)*XX(144)-JVS(1712)&
              &*XX(145)-JVS(1770)*XX(146)
  XX(121) = XX(121)-JVS(1213)*XX(136)-JVS(1352)*XX(139)-JVS(1450)*XX(141)-JVS(1573)*XX(143)-JVS(1657)*XX(144)-JVS(1769)&
              &*XX(146)
  XX(120) = XX(120)-JVS(777)*XX(121)-JVS(879)*XX(125)-JVS(907)*XX(126)-JVS(969)*XX(129)-JVS(1025)*XX(131)-JVS(1212)&
              &*XX(136)-JVS(1302)*XX(138)-JVS(1351)*XX(139)-JVS(1380)*XX(140)-JVS(1449)*XX(141)-JVS(1479)*XX(142)-JVS(1572)&
              &*XX(143)-JVS(1656)*XX(144)-JVS(1711)*XX(145)-JVS(1768)*XX(146)
  XX(119) = XX(119)-JVS(878)*XX(125)-JVS(968)*XX(129)-JVS(1093)*XX(133)-JVS(1163)*XX(135)-JVS(1211)*XX(136)-JVS(1250)&
              &*XX(137)-JVS(1301)*XX(138)-JVS(1350)*XX(139)-JVS(1448)*XX(141)-JVS(1571)*XX(143)-JVS(1655)*XX(144)-JVS(1710)&
              &*XX(145)-JVS(1767)*XX(146)
  XX(118) = XX(118)-JVS(727)*XX(119)-JVS(815)*XX(123)-JVS(877)*XX(125)-JVS(967)*XX(129)-JVS(1024)*XX(131)-JVS(1055)&
              &*XX(132)-JVS(1092)*XX(133)-JVS(1162)*XX(135)-JVS(1210)*XX(136)-JVS(1249)*XX(137)-JVS(1300)*XX(138)-JVS(1349)&
              &*XX(139)-JVS(1447)*XX(141)-JVS(1570)*XX(143)-JVS(1654)*XX(144)-JVS(1709)*XX(145)-JVS(1766)*XX(146)
  XX(117) = XX(117)-JVS(876)*XX(125)-JVS(997)*XX(130)-JVS(1023)*XX(131)-JVS(1054)*XX(132)-JVS(1091)*XX(133)-JVS(1299)&
              &*XX(138)-JVS(1348)*XX(139)-JVS(1446)*XX(141)-JVS(1569)*XX(143)-JVS(1653)*XX(144)-JVS(1708)*XX(145)
  XX(116) = XX(116)-JVS(776)*XX(121)-JVS(875)*XX(125)-JVS(966)*XX(129)-JVS(1053)*XX(132)-JVS(1161)*XX(135)-JVS(1298)&
              &*XX(138)-JVS(1347)*XX(139)-JVS(1445)*XX(141)-JVS(1568)*XX(143)-JVS(1652)*XX(144)-JVS(1707)*XX(145)
  XX(115) = XX(115)-JVS(874)*XX(125)-JVS(1022)*XX(131)-JVS(1248)*XX(137)-JVS(1297)*XX(138)-JVS(1346)*XX(139)-JVS(1444)&
              &*XX(141)-JVS(1567)*XX(143)-JVS(1651)*XX(144)-JVS(1706)*XX(145)
  XX(114) = XX(114)-JVS(873)*XX(125)-JVS(1209)*XX(136)-JVS(1443)*XX(141)-JVS(1566)*XX(143)-JVS(1650)*XX(144)-JVS(1765)&
              &*XX(146)
  XX(113) = XX(113)-JVS(872)*XX(125)-JVS(1021)*XX(131)-JVS(1296)*XX(138)-JVS(1345)*XX(139)-JVS(1442)*XX(141)-JVS(1565)&
              &*XX(143)-JVS(1649)*XX(144)-JVS(1705)*XX(145)
  XX(112) = XX(112)-JVS(655)*XX(114)-JVS(775)*XX(121)-JVS(871)*XX(125)-JVS(965)*XX(129)-JVS(996)*XX(130)-JVS(1090)&
              &*XX(133)-JVS(1160)*XX(135)-JVS(1208)*XX(136)-JVS(1247)*XX(137)-JVS(1295)*XX(138)-JVS(1344)*XX(139)-JVS(1441)&
              &*XX(141)-JVS(1564)*XX(143)-JVS(1648)*XX(144)-JVS(1704)*XX(145)-JVS(1764)*XX(146)
  XX(111) = XX(111)-JVS(654)*XX(114)-JVS(774)*XX(121)-JVS(870)*XX(125)-JVS(1089)*XX(133)-JVS(1159)*XX(135)-JVS(1294)&
              &*XX(138)-JVS(1343)*XX(139)-JVS(1440)*XX(141)-JVS(1563)*XX(143)-JVS(1647)*XX(144)-JVS(1703)*XX(145)-JVS(1763)&
              &*XX(146)
  XX(110) = XX(110)-JVS(691)*XX(116)-JVS(773)*XX(121)-JVS(869)*XX(125)-JVS(1020)*XX(131)-JVS(1052)*XX(132)-JVS(1088)&
              &*XX(133)-JVS(1158)*XX(135)-JVS(1293)*XX(138)-JVS(1342)*XX(139)-JVS(1439)*XX(141)-JVS(1562)*XX(143)-JVS(1646)&
              &*XX(144)-JVS(1702)*XX(145)
  XX(109) = XX(109)-JVS(772)*XX(121)-JVS(868)*XX(125)-JVS(1019)*XX(131)-JVS(1087)*XX(133)-JVS(1207)*XX(136)-JVS(1292)&
              &*XX(138)-JVS(1341)*XX(139)-JVS(1438)*XX(141)-JVS(1561)*XX(143)-JVS(1645)*XX(144)-JVS(1701)*XX(145)
  XX(108) = XX(108)-JVS(690)*XX(116)-JVS(867)*XX(125)-JVS(1086)*XX(133)-JVS(1128)*XX(134)-JVS(1206)*XX(136)-JVS(1291)&
              &*XX(138)-JVS(1340)*XX(139)-JVS(1437)*XX(141)-JVS(1560)*XX(143)-JVS(1644)*XX(144)-JVS(1700)*XX(145)
  XX(107) = XX(107)-JVS(704)*XX(117)-JVS(771)*XX(121)-JVS(866)*XX(125)-JVS(923)*XX(127)-JVS(937)*XX(128)-JVS(1051)&
              &*XX(132)-JVS(1085)*XX(133)-JVS(1205)*XX(136)-JVS(1290)*XX(138)-JVS(1339)*XX(139)-JVS(1436)*XX(141)-JVS(1559)&
              &*XX(143)-JVS(1643)*XX(144)-JVS(1699)*XX(145)
  XX(106) = XX(106)-JVS(675)*XX(115)-JVS(726)*XX(119)-JVS(865)*XX(125)-JVS(964)*XX(129)-JVS(1018)*XX(131)-JVS(1050)&
              &*XX(132)-JVS(1204)*XX(136)-JVS(1246)*XX(137)-JVS(1289)*XX(138)-JVS(1338)*XX(139)-JVS(1435)*XX(141)-JVS(1558)&
              &*XX(143)-JVS(1642)*XX(144)-JVS(1762)*XX(146)
  XX(105) = XX(105)-JVS(864)*XX(125)-JVS(963)*XX(129)-JVS(1017)*XX(131)-JVS(1084)*XX(133)-JVS(1288)*XX(138)-JVS(1337)&
              &*XX(139)-JVS(1434)*XX(141)-JVS(1557)*XX(143)-JVS(1641)*XX(144)-JVS(1698)*XX(145)
  XX(104) = XX(104)-JVS(748)*XX(120)-JVS(863)*XX(125)-JVS(906)*XX(126)-JVS(962)*XX(129)-JVS(1203)*XX(136)-JVS(1287)&
              &*XX(138)-JVS(1433)*XX(141)-JVS(1478)*XX(142)-JVS(1556)*XX(143)-JVS(1640)*XX(144)-JVS(1761)*XX(146)
  XX(103) = XX(103)-JVS(770)*XX(121)-JVS(862)*XX(125)-JVS(961)*XX(129)-JVS(1202)*XX(136)-JVS(1286)*XX(138)-JVS(1432)&
              &*XX(141)-JVS(1555)*XX(143)-JVS(1639)*XX(144)-JVS(1760)*XX(146)
  XX(102) = XX(102)-JVS(653)*XX(114)-JVS(769)*XX(121)-JVS(826)*XX(124)-JVS(861)*XX(125)-JVS(936)*XX(128)-JVS(960)&
              &*XX(129)-JVS(1201)*XX(136)-JVS(1245)*XX(137)-JVS(1431)*XX(141)-JVS(1554)*XX(143)-JVS(1638)*XX(144)-JVS(1697)&
              &*XX(145)
  XX(101) = XX(101)-JVS(860)*XX(125)-JVS(959)*XX(129)-JVS(1083)*XX(133)-JVS(1157)*XX(135)-JVS(1285)*XX(138)-JVS(1336)&
              &*XX(139)-JVS(1430)*XX(141)-JVS(1553)*XX(143)-JVS(1637)*XX(144)-JVS(1696)*XX(145)-JVS(1759)*XX(146)
  XX(100) = XX(100)-JVS(859)*XX(125)-JVS(1200)*XX(136)-JVS(1429)*XX(141)-JVS(1552)*XX(143)-JVS(1636)*XX(144)-JVS(1758)&
              &*XX(146)
  XX(99) = XX(99)-JVS(858)*XX(125)-JVS(1082)*XX(133)-JVS(1284)*XX(138)-JVS(1335)*XX(139)-JVS(1428)*XX(141)-JVS(1551)&
             &*XX(143)-JVS(1635)*XX(144)-JVS(1695)*XX(145)
  XX(98) = XX(98)-JVS(1427)*XX(141)-JVS(1550)*XX(143)-JVS(1757)*XX(146)
  XX(97) = XX(97)-JVS(511)*XX(100)-JVS(814)*XX(123)-JVS(857)*XX(125)-JVS(1049)*XX(132)-JVS(1081)*XX(133)-JVS(1156)&
             &*XX(135)-JVS(1283)*XX(138)-JVS(1426)*XX(141)-JVS(1549)*XX(143)-JVS(1634)*XX(144)-JVS(1694)*XX(145)-JVS(1756)&
             &*XX(146)
  XX(96) = XX(96)-JVS(747)*XX(120)-JVS(768)*XX(121)-JVS(1199)*XX(136)-JVS(1282)*XX(138)-JVS(1334)*XX(139)-JVS(1425)&
             &*XX(141)-JVS(1548)*XX(143)-JVS(1633)*XX(144)-JVS(1693)*XX(145)-JVS(1755)*XX(146)
  XX(95) = XX(95)-JVS(767)*XX(121)-JVS(856)*XX(125)-JVS(1198)*XX(136)-JVS(1424)*XX(141)-JVS(1547)*XX(143)-JVS(1692)&
             &*XX(145)
  XX(94) = XX(94)-JVS(746)*XX(120)-JVS(766)*XX(121)-JVS(1197)*XX(136)-JVS(1281)*XX(138)-JVS(1423)*XX(141)-JVS(1546)&
             &*XX(143)-JVS(1632)*XX(144)-JVS(1754)*XX(146)
  XX(93) = XX(93)-JVS(474)*XX(98)-JVS(1127)*XX(134)-JVS(1244)*XX(137)-JVS(1280)*XX(138)-JVS(1422)*XX(141)-JVS(1545)&
             &*XX(143)-JVS(1631)*XX(144)-JVS(1753)*XX(146)
  XX(92) = XX(92)-JVS(559)*XX(105)-JVS(595)*XX(109)-JVS(745)*XX(120)-JVS(855)*XX(125)-JVS(1016)*XX(131)-JVS(1196)&
             &*XX(136)-JVS(1243)*XX(137)-JVS(1279)*XX(138)-JVS(1421)*XX(141)-JVS(1544)*XX(143)-JVS(1630)*XX(144)-JVS(1691)&
             &*XX(145)-JVS(1752)*XX(146)
  XX(91) = XX(91)-JVS(419)*XX(92)-JVS(549)*XX(104)-JVS(614)*XX(111)-JVS(744)*XX(120)-JVS(802)*XX(122)-JVS(813)*XX(123)&
             &-JVS(854)*XX(125)-JVS(1195)*XX(136)-JVS(1242)*XX(137)-JVS(1278)*XX(138)-JVS(1333)*XX(139)-JVS(1379)*XX(140)&
             &-JVS(1420)*XX(141)-JVS(1477)*XX(142)-JVS(1543)*XX(143)-JVS(1629)*XX(144)-JVS(1690)*XX(145)-JVS(1751)*XX(146)
  XX(90) = XX(90)-JVS(743)*XX(120)-JVS(853)*XX(125)-JVS(1277)*XX(138)-JVS(1419)*XX(141)-JVS(1542)*XX(143)-JVS(1628)&
             &*XX(144)-JVS(1750)*XX(146)
  XX(89) = XX(89)-JVS(510)*XX(100)-JVS(812)*XX(123)-JVS(852)*XX(125)-JVS(1048)*XX(132)-JVS(1080)*XX(133)-JVS(1155)&
             &*XX(135)-JVS(1276)*XX(138)-JVS(1418)*XX(141)-JVS(1541)*XX(143)-JVS(1627)*XX(144)-JVS(1689)*XX(145)
  XX(88) = XX(88)-JVS(689)*XX(116)-JVS(1126)*XX(134)-JVS(1194)*XX(136)-JVS(1332)*XX(139)-JVS(1540)*XX(143)-JVS(1688)&
             &*XX(145)
  XX(87) = XX(87)-JVS(652)*XX(114)-JVS(742)*XX(120)-JVS(958)*XX(129)-JVS(1193)*XX(136)-JVS(1275)*XX(138)-JVS(1417)&
             &*XX(141)-JVS(1539)*XX(143)-JVS(1626)*XX(144)-JVS(1749)*XX(146)
  XX(86) = XX(86)-JVS(651)*XX(114)-JVS(765)*XX(121)-JVS(851)*XX(125)-JVS(1331)*XX(139)-JVS(1416)*XX(141)-JVS(1538)&
             &*XX(143)-JVS(1748)*XX(146)
  XX(85) = XX(85)-JVS(741)*XX(120)-JVS(1015)*XX(131)-JVS(1192)*XX(136)-JVS(1274)*XX(138)-JVS(1415)*XX(141)-JVS(1537)&
             &*XX(143)-JVS(1625)*XX(144)-JVS(1747)*XX(146)
  XX(84) = XX(84)-JVS(740)*XX(120)-JVS(1191)*XX(136)-JVS(1273)*XX(138)-JVS(1378)*XX(140)-JVS(1414)*XX(141)-JVS(1536)&
             &*XX(143)-JVS(1624)*XX(144)-JVS(1746)*XX(146)
  XX(83) = XX(83)-JVS(725)*XX(119)-JVS(850)*XX(125)-JVS(957)*XX(129)-JVS(1190)*XX(136)-JVS(1241)*XX(137)-JVS(1330)&
             &*XX(139)-JVS(1413)*XX(141)-JVS(1535)*XX(143)-JVS(1745)*XX(146)
  XX(82) = XX(82)-JVS(473)*XX(98)-JVS(509)*XX(100)-JVS(650)*XX(114)-JVS(764)*XX(121)-JVS(956)*XX(129)-JVS(995)*XX(130)&
             &-JVS(1079)*XX(133)-JVS(1272)*XX(138)-JVS(1412)*XX(141)-JVS(1534)*XX(143)-JVS(1623)*XX(144)
  XX(81) = XX(81)-JVS(1078)*XX(133)-JVS(1533)*XX(143)-JVS(1622)*XX(144)
  XX(80) = XX(80)-JVS(472)*XX(98)-JVS(994)*XX(130)-JVS(1077)*XX(133)-JVS(1271)*XX(138)-JVS(1411)*XX(141)-JVS(1532)&
             &*XX(143)-JVS(1621)*XX(144)
  XX(79) = XX(79)-JVS(763)*XX(121)-JVS(849)*XX(125)-JVS(955)*XX(129)-JVS(1270)*XX(138)-JVS(1410)*XX(141)-JVS(1531)&
             &*XX(143)-JVS(1620)*XX(144)-JVS(1744)*XX(146)
  XX(78) = XX(78)-JVS(508)*XX(100)-JVS(649)*XX(114)-JVS(762)*XX(121)-JVS(848)*XX(125)-JVS(954)*XX(129)-JVS(1189)*XX(136)&
             &-JVS(1269)*XX(138)-JVS(1409)*XX(141)-JVS(1530)*XX(143)-JVS(1619)*XX(144)
  XX(77) = XX(77)-JVS(373)*XX(86)-JVS(539)*XX(103)-JVS(825)*XX(124)-JVS(847)*XX(125)-JVS(922)*XX(127)-JVS(1240)*XX(137)&
             &-JVS(1408)*XX(141)-JVS(1529)*XX(143)-JVS(1618)*XX(144)
  XX(76) = XX(76)-JVS(703)*XX(117)-JVS(921)*XX(127)-JVS(935)*XX(128)-JVS(1076)*XX(133)-JVS(1407)*XX(141)-JVS(1528)&
             &*XX(143)-JVS(1617)*XX(144)
  XX(75) = XX(75)-JVS(811)*XX(123)-JVS(1047)*XX(132)-JVS(1075)*XX(133)-JVS(1154)*XX(135)-JVS(1406)*XX(141)-JVS(1527)&
             &*XX(143)-JVS(1616)*XX(144)
  XX(74) = XX(74)-JVS(325)*XX(80)-JVS(1153)*XX(135)-JVS(1239)*XX(137)-JVS(1526)*XX(143)-JVS(1615)*XX(144)
  XX(73) = XX(73)-JVS(953)*XX(129)-JVS(1152)*XX(135)-JVS(1188)*XX(136)-JVS(1525)*XX(143)-JVS(1614)*XX(144)
  XX(72) = XX(72)-JVS(281)*XX(73)-JVS(624)*XX(112)-JVS(648)*XX(114)-JVS(761)*XX(121)-JVS(952)*XX(129)-JVS(1187)*XX(136)&
             &-JVS(1524)*XX(143)-JVS(1613)*XX(144)
  XX(71) = XX(71)-JVS(846)*XX(125)-JVS(905)*XX(126)-JVS(951)*XX(129)-JVS(1186)*XX(136)-JVS(1523)*XX(143)-JVS(1612)&
             &*XX(144)
  XX(70) = XX(70)-JVS(312)*XX(78)-JVS(801)*XX(122)-JVS(824)*XX(124)-JVS(845)*XX(125)-JVS(1377)*XX(140)-JVS(1476)*XX(142)&
             &-JVS(1522)*XX(143)-JVS(1611)*XX(144)
  XX(69) = XX(69)-JVS(613)*XX(111)-JVS(647)*XX(114)-JVS(760)*XX(121)-JVS(844)*XX(125)-JVS(1151)*XX(135)-JVS(1329)&
             &*XX(139)-JVS(1521)*XX(143)-JVS(1610)*XX(144)
  XX(68) = XX(68)-JVS(688)*XX(116)-JVS(759)*XX(121)-JVS(843)*XX(125)-JVS(1328)*XX(139)-JVS(1520)*XX(143)
  XX(67) = XX(67)-JVS(993)*XX(130)-JVS(1074)*XX(133)-JVS(1519)*XX(143)-JVS(1609)*XX(144)
  XX(66) = XX(66)-JVS(1518)*XX(143)-JVS(1608)*XX(144)
  XX(65) = XX(65)-JVS(558)*XX(105)-JVS(842)*XX(125)-JVS(950)*XX(129)-JVS(1014)*XX(131)-JVS(1517)*XX(143)-JVS(1607)&
             &*XX(144)
  XX(64) = XX(64)-JVS(471)*XX(98)-JVS(633)*XX(113)-JVS(1125)*XX(134)-JVS(1185)*XX(136)-JVS(1516)*XX(143)-JVS(1743)&
             &*XX(146)
  XX(63) = XX(63)-JVS(841)*XX(125)-JVS(1405)*XX(141)-JVS(1515)*XX(143)
  XX(62) = XX(62)-JVS(425)*XX(93)-JVS(470)*XX(98)-JVS(1124)*XX(134)-JVS(1404)*XX(141)-JVS(1514)*XX(143)-JVS(1742)&
             &*XX(146)
  XX(61) = XX(61)-JVS(442)*XX(95)-JVS(594)*XX(109)-JVS(1073)*XX(133)-JVS(1403)*XX(141)-JVS(1513)*XX(143)-JVS(1606)&
             &*XX(144)
  XX(60) = XX(60)-JVS(674)*XX(115)-JVS(1013)*XX(131)-JVS(1512)*XX(143)-JVS(1605)*XX(144)
  XX(59) = XX(59)-JVS(524)*XX(101)-JVS(724)*XX(119)-JVS(840)*XX(125)-JVS(1327)*XX(139)-JVS(1511)*XX(143)
  XX(58) = XX(58)-JVS(1123)*XX(134)-JVS(1510)*XX(143)
  XX(57) = XX(57)-JVS(1122)*XX(134)-JVS(1509)*XX(143)
  XX(56) = XX(56)-JVS(194)*XX(57)-JVS(1121)*XX(134)-JVS(1508)*XX(143)
  XX(55) = XX(55)-JVS(1402)*XX(141)-JVS(1507)*XX(143)-JVS(1604)*XX(144)-JVS(1741)*XX(146)
  XX(54) = XX(54)-JVS(632)*XX(113)-JVS(1012)*XX(131)-JVS(1506)*XX(143)-JVS(1603)*XX(144)
  XX(53) = XX(53)-JVS(202)*XX(58)-JVS(498)*XX(99)-JVS(584)*XX(108)-JVS(1120)*XX(134)-JVS(1184)*XX(136)-JVS(1505)*XX(143)
  XX(52) = XX(52)-JVS(839)*XX(125)-JVS(1504)*XX(143)-JVS(1602)*XX(144)-JVS(1687)*XX(145)
  XX(51) = XX(51)-JVS(469)*XX(98)-JVS(739)*XX(120)-JVS(838)*XX(125)-JVS(1503)*XX(143)-JVS(1686)*XX(145)-JVS(1740)&
             &*XX(146)
  XX(50) = XX(50)-JVS(389)*XX(88)-JVS(583)*XX(108)-JVS(1502)*XX(143)-JVS(1601)*XX(144)
  XX(49) = XX(49)-JVS(497)*XX(99)-JVS(1072)*XX(133)-JVS(1501)*XX(143)-JVS(1600)*XX(144)
  XX(48) = XX(48)-JVS(837)*XX(125)-JVS(1401)*XX(141)-JVS(1599)*XX(144)-JVS(1685)*XX(145)-JVS(1739)*XX(146)
  XX(47) = XX(47)-JVS(1268)*XX(138)-JVS(1400)*XX(141)-JVS(1500)*XX(143)
  XX(46) = XX(46)-JVS(1326)*XX(139)-JVS(1499)*XX(143)-JVS(1684)*XX(145)
  XX(45) = XX(45)-JVS(468)*XX(98)-JVS(992)*XX(130)-JVS(1498)*XX(143)-JVS(1738)*XX(146)
  XX(44) = XX(44)-JVS(1325)*XX(139)-JVS(1399)*XX(141)-JVS(1683)*XX(145)-JVS(1737)*XX(146)
  XX(43) = XX(43)-JVS(467)*XX(98)-JVS(1398)*XX(141)-JVS(1736)*XX(146)
  XX(42) = XX(42)-JVS(318)*XX(79)-JVS(1397)*XX(141)
  XX(41) = XX(41)-JVS(311)*XX(78)-JVS(1497)*XX(143)
  XX(40) = XX(40)-JVS(1119)*XX(134)-JVS(1396)*XX(141)
  XX(39) = XX(39)-JVS(673)*XX(115)-JVS(1395)*XX(141)
  XX(38) = XX(38)-JVS(1118)*XX(134)-JVS(1496)*XX(143)
  XX(37) = XX(37)-JVS(1117)*XX(134)-JVS(1495)*XX(143)
  XX(36) = XX(36)-JVS(1116)*XX(134)-JVS(1494)*XX(143)
  XX(35) = XX(35)
  XX(34) = XX(34)
  XX(33) = XX(33)
  XX(32) = XX(32)
  XX(31) = XX(31)
  XX(30) = XX(30)
  XX(29) = XX(29)
  XX(28) = XX(28)
  XX(27) = XX(27)
  XX(26) = XX(26)
  XX(25) = XX(25)
  XX(24) = XX(24)
  XX(23) = XX(23)
  XX(22) = XX(22)
  XX(21) = XX(21)
  XX(20) = XX(20)
  XX(19) = XX(19)
  XX(18) = XX(18)
  XX(17) = XX(17)
  XX(16) = XX(16)
  XX(15) = XX(15)
  XX(14) = XX(14)
  XX(13) = XX(13)
  XX(12) = XX(12)
  XX(11) = XX(11)
  XX(10) = XX(10)
  XX(9) = XX(9)
  XX(8) = XX(8)
  XX(7) = XX(7)
  XX(6) = XX(6)
  XX(5) = XX(5)
  XX(4) = XX(4)
  XX(3) = XX(3)
  XX(2) = XX(2)
  XX(1) = XX(1)
      
END SUBROUTINE KppSolveTR

! End of KppSolveTR function
! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~


! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
! 
! BLAS_UTIL - BLAS-LIKE utility functions
!   Arguments :
! 
! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

!--------------------------------------------------------------
!
! BLAS/LAPACK-like subroutines used by the integration algorithms
! It is recommended to replace them by calls to the optimized
!      BLAS/LAPACK library for your machine
!
!  (C) Adrian Sandu, Aug. 2004
!      Virginia Polytechnic Institute and State University
!--------------------------------------------------------------


!--------------------------------------------------------------
      SUBROUTINE WCOPY(N,X,incX,Y,incY)
!--------------------------------------------------------------
!     copies a vector, x, to a vector, y:  y <- x
!     only for incX=incY=1
!     after BLAS
!     replace this by the function from the optimized BLAS implementation:
!         CALL  SCOPY(N,X,1,Y,1)   or   CALL  DCOPY(N,X,1,Y,1)
!--------------------------------------------------------------
!     USE gckpp_Precision
      
      INTEGER  :: i,incX,incY,M,MP1,N
      REAL(kind=dp) :: X(N),Y(N)

      IF (N.LE.0) RETURN

      M = MOD(N,8)
      IF( M .NE. 0 ) THEN
        DO i = 1,M
          Y(i) = X(i)
        END DO
        IF( N .LT. 8 ) RETURN
      END IF    
      MP1 = M+1
      DO i = MP1,N,8
        Y(i) = X(i)
        Y(i + 1) = X(i + 1)
        Y(i + 2) = X(i + 2)
        Y(i + 3) = X(i + 3)
        Y(i + 4) = X(i + 4)
        Y(i + 5) = X(i + 5)
        Y(i + 6) = X(i + 6)
        Y(i + 7) = X(i + 7)
      END DO

      END SUBROUTINE WCOPY


!--------------------------------------------------------------
      SUBROUTINE WAXPY(N,Alpha,X,incX,Y,incY)
!--------------------------------------------------------------
!     constant times a vector plus a vector: y <- y + Alpha*x
!     only for incX=incY=1
!     after BLAS
!     replace this by the function from the optimized BLAS implementation:
!         CALL SAXPY(N,Alpha,X,1,Y,1) or  CALL DAXPY(N,Alpha,X,1,Y,1)
!--------------------------------------------------------------

      INTEGER  :: i,incX,incY,M,MP1,N
      REAL(kind=dp) :: X(N),Y(N),Alpha
      REAL(kind=dp), PARAMETER :: ZERO = 0.0_dp

      IF (Alpha .EQ. ZERO) RETURN
      IF (N .LE. 0) RETURN

      M = MOD(N,4)
      IF( M .NE. 0 ) THEN
        DO i = 1,M
          Y(i) = Y(i) + Alpha*X(i)
        END DO
        IF( N .LT. 4 ) RETURN
      END IF
      MP1 = M + 1
      DO i = MP1,N,4
        Y(i) = Y(i) + Alpha*X(i)
        Y(i + 1) = Y(i + 1) + Alpha*X(i + 1)
        Y(i + 2) = Y(i + 2) + Alpha*X(i + 2)
        Y(i + 3) = Y(i + 3) + Alpha*X(i + 3)
      END DO
      
      END SUBROUTINE WAXPY



!--------------------------------------------------------------
      SUBROUTINE WSCAL(N,Alpha,X,incX)
!--------------------------------------------------------------
!     constant times a vector: x(1:N) <- Alpha*x(1:N) 
!     only for incX=incY=1
!     after BLAS
!     replace this by the function from the optimized BLAS implementation:
!         CALL SSCAL(N,Alpha,X,1) or  CALL DSCAL(N,Alpha,X,1)
!--------------------------------------------------------------

      INTEGER  :: i,incX,M,MP1,N
      REAL(kind=dp)  :: X(N),Alpha
      REAL(kind=dp), PARAMETER  :: ZERO=0.0_dp, ONE=1.0_dp

      IF (Alpha .EQ. ONE) RETURN
      IF (N .LE. 0) RETURN

      M = MOD(N,5)
      IF( M .NE. 0 ) THEN
        IF (Alpha .EQ. (-ONE)) THEN
          DO i = 1,M
            X(i) = -X(i)
          END DO
        ELSEIF (Alpha .EQ. ZERO) THEN
          DO i = 1,M
            X(i) = ZERO
          END DO
        ELSE
          DO i = 1,M
            X(i) = Alpha*X(i)
          END DO
        END IF
        IF( N .LT. 5 ) RETURN
      END IF
      MP1 = M + 1
      IF (Alpha .EQ. (-ONE)) THEN
        DO i = MP1,N,5
          X(i)     = -X(i)
          X(i + 1) = -X(i + 1)
          X(i + 2) = -X(i + 2)
          X(i + 3) = -X(i + 3)
          X(i + 4) = -X(i + 4)
        END DO
      ELSEIF (Alpha .EQ. ZERO) THEN
        DO i = MP1,N,5
          X(i)     = ZERO
          X(i + 1) = ZERO
          X(i + 2) = ZERO
          X(i + 3) = ZERO
          X(i + 4) = ZERO
        END DO
      ELSE
        DO i = MP1,N,5
          X(i)     = Alpha*X(i)
          X(i + 1) = Alpha*X(i + 1)
          X(i + 2) = Alpha*X(i + 2)
          X(i + 3) = Alpha*X(i + 3)
          X(i + 4) = Alpha*X(i + 4)
        END DO
      END IF

      END SUBROUTINE WSCAL

!--------------------------------------------------------------
      REAL(kind=dp) FUNCTION WLAMCH( C )
!--------------------------------------------------------------
!     returns epsilon machine
!     after LAPACK
!     replace this by the function from the optimized LAPACK implementation:
!          CALL SLAMCH('E') or CALL DLAMCH('E')
!--------------------------------------------------------------
!      USE gckpp_Precision

      CHARACTER ::  C
      INTEGER    :: i
      REAL(kind=dp), SAVE  ::  Eps
      REAL(kind=dp)  ::  Suma
      REAL(kind=dp), PARAMETER  ::  ONE=1.0_dp, HALF=0.5_dp
      LOGICAL, SAVE   ::  First=.TRUE.
      
!$OMP THREADPRIVATE( Eps, First )

      IF (First) THEN
        First = .FALSE.
        Eps = HALF**(16)
        DO i = 17, 80
          Eps = Eps*HALF
          CALL WLAMCH_ADD(ONE,Eps,Suma)
          IF (Suma.LE.ONE) GOTO 10
        END DO
        PRINT*,'ERROR IN WLAMCH. EPS < ',Eps
        RETURN
10      Eps = Eps*2
        i = i-1      
      END IF

      WLAMCH = Eps

      END FUNCTION WLAMCH
     
      SUBROUTINE WLAMCH_ADD( A, B, Suma )
!      USE gckpp_Precision
      
      REAL(kind=dp) A, B, Suma
      Suma = A + B

      END SUBROUTINE WLAMCH_ADD
!--------------------------------------------------------------


!--------------------------------------------------------------
      SUBROUTINE SET2ZERO(N,Y)
!--------------------------------------------------------------
!     copies zeros into the vector y:  y <- 0
!     after BLAS
!--------------------------------------------------------------
      
      INTEGER ::  i,M,MP1,N
      REAL(kind=dp) ::  Y(N)
      REAL(kind=dp), PARAMETER :: ZERO = 0.0d0

      IF (N.LE.0) RETURN

      M = MOD(N,8)
      IF( M .NE. 0 ) THEN
        DO i = 1,M
          Y(i) = ZERO
        END DO
        IF( N .LT. 8 ) RETURN
      END IF    
      MP1 = M+1
      DO i = MP1,N,8
        Y(i)     = ZERO
        Y(i + 1) = ZERO
        Y(i + 2) = ZERO
        Y(i + 3) = ZERO
        Y(i + 4) = ZERO
        Y(i + 5) = ZERO
        Y(i + 6) = ZERO
        Y(i + 7) = ZERO
      END DO

      END SUBROUTINE SET2ZERO


!--------------------------------------------------------------
      REAL(kind=dp) FUNCTION WDOT (N, DX, incX, DY, incY) 
!--------------------------------------------------------------
!     dot produce: wdot = x(1:N)*y(1:N) 
!     only for incX=incY=1
!     after BLAS
!     replace this by the function from the optimized BLAS implementation:
!         CALL SDOT(N,X,1,Y,1) or  CALL DDOT(N,X,1,Y,1)
!--------------------------------------------------------------
!      USE messy_mecca_kpp_Precision
!--------------------------------------------------------------
      IMPLICIT NONE
      INTEGER :: N, incX, incY
      REAL(kind=dp) :: DX(N), DY(N) 

      INTEGER :: i, IX, IY, M, MP1, NS
                                 
      WDOT = 0.0D0 
      IF (N .LE. 0) RETURN 
      IF (incX .EQ. incY) IF (incX-1) 5,20,60 
!                                                                       
!     Code for unequal or nonpositive increments.                       
!                                                                       
    5 IX = 1 
      IY = 1 
      IF (incX .LT. 0) IX = (-N+1)*incX + 1 
      IF (incY .LT. 0) IY = (-N+1)*incY + 1 
      DO i = 1,N 
        WDOT = WDOT + DX(IX)*DY(IY) 
        IX = IX + incX 
        IY = IY + incY 
      END DO 
      RETURN 
!                                                                       
!     Code for both increments equal to 1.                              
!                                                                       
!     Clean-up loop so remaining vector length is a multiple of 5.      
!                                                                       
   20 M = MOD(N,5) 
      IF (M .EQ. 0) GO TO 40 
      DO i = 1,M 
         WDOT = WDOT + DX(i)*DY(i) 
      END DO 
      IF (N .LT. 5) RETURN 
   40 MP1 = M + 1 
      DO i = MP1,N,5 
          WDOT = WDOT + DX(i)*DY(i) + DX(i+1)*DY(i+1) + DX(i+2)*DY(i+2) +  &
                   DX(i+3)*DY(i+3) + DX(i+4)*DY(i+4)                   
      END DO 
      RETURN 
!                                                                       
!     Code for equal, positive, non-unit increments.                    
!                                                                       
   60 NS = N*incX 
      DO i = 1,NS,incX 
        WDOT = WDOT + DX(i)*DY(i) 
      END DO 

      END FUNCTION WDOT                                          


!--------------------------------------------------------------
      SUBROUTINE WADD(N,X,Y,Z)
!--------------------------------------------------------------
!     adds two vectors: z <- x + y
!     BLAS - like
!--------------------------------------------------------------
!     USE gckpp_Precision
      
      INTEGER :: i, M, MP1, N
      REAL(kind=dp) :: X(N),Y(N),Z(N)

      IF (N.LE.0) RETURN

      M = MOD(N,5)
      IF( M /= 0 ) THEN
         DO i = 1,M
            Z(i) = X(i) + Y(i)
         END DO
         IF( N < 5 ) RETURN
      END IF    
      MP1 = M+1
      DO i = MP1,N,5
         Z(i)     = X(i)     + Y(i)
         Z(i + 1) = X(i + 1) + Y(i + 1)
         Z(i + 2) = X(i + 2) + Y(i + 2)
         Z(i + 3) = X(i + 3) + Y(i + 3)
         Z(i + 4) = X(i + 4) + Y(i + 4)
      END DO

      END SUBROUTINE WADD
      
      
      
!--------------------------------------------------------------
      SUBROUTINE WGEFA(N,A,Ipvt,info)
!--------------------------------------------------------------
!     WGEFA FACTORS THE MATRIX A (N,N) BY
!           GAUSS ELIMINATION WITH PARTIAL PIVOTING
!     LINPACK - LIKE 
!--------------------------------------------------------------
!
      INTEGER       :: N,Ipvt(N),info
      REAL(kind=dp) :: A(N,N)
      REAL(kind=dp) :: t, dmax, da
      INTEGER       :: j,k,l
      REAL(kind=dp), PARAMETER :: ZERO = 0.0, ONE = 1.0

      info = 0

size: IF (n > 1) THEN
      
col:  DO k = 1, n-1

!        find l = pivot index
!        l = idamax(n-k+1,A(k,k),1) + k - 1
         l = k; dmax = abs(A(k,k))
         DO j = k+1,n
            da = ABS(A(j,k))
            IF (da > dmax) THEN
              l = j; dmax = da
            END IF
         END DO
         Ipvt(k) = l

!        zero pivot implies this column already triangularized
         IF (ABS(A(l,k)) < TINY(ZERO)) THEN
            info = k
            return
         ELSE   
            IF (l /= k) THEN
               t = A(l,k); A(l,k) = A(k,k); A(k,k) = t
            END IF
            t = -ONE/A(k,k)
            CALL WSCAL(n-k,t,A(k+1,k),1)
            DO j = k+1, n
               t = A(l,j)
               IF (l /= k) THEN
                  A(l,j) = A(k,j); A(k,j) = t
               END IF
               CALL WAXPY(n-k,t,A(k+1,k),1,A(k+1,j),1)
            END DO         
         END IF
         
       END DO col
       
      END IF size
      
      Ipvt(N) = N
      IF (ABS(A(N,N)) == ZERO) info = N
      
      END SUBROUTINE WGEFA


!--------------------------------------------------------------
      SUBROUTINE WGESL(Trans,N,A,Ipvt,b)
!--------------------------------------------------------------
!     WGESL solves the system
!     a * x = b  or  trans(a) * x = b
!     using the factors computed by WGEFA.
!
!     Trans      = 'N'   to solve  A*x = b ,
!                = 'T'   to solve  transpose(A)*x = b
!     LINPACK - LIKE 
!--------------------------------------------------------------

      INTEGER       :: N,Ipvt(N)
      CHARACTER     :: trans
      REAL(kind=dp) :: A(N,N),b(N)
      REAL(kind=dp) :: t
      INTEGER       :: k,kb,l

      
      SELECT CASE (Trans)

      CASE ('n','N')  !  Solve  A * x = b

!        first solve  L*y = b
         IF (n >= 2) THEN
          DO k = 1, n-1
            l = Ipvt(k)
            t = b(l)
            IF (l /= k) THEN
               b(l) = b(k)
               b(k) = t
            END IF
            CALL WAXPY(n-k,t,a(k+1,k),1,b(k+1),1)
          END DO
         END IF
!        now solve  U*x = y
         DO kb = 1, n
            k = n + 1 - kb
            b(k) = b(k)/a(k,k)
            t = -b(k)
            CALL WAXPY(k-1,t,a(1,k),1,b(1),1)
         END DO
      
      CASE ('t','T')  !  Solve transpose(A) * x = b

!        first solve  trans(U)*y = b
         DO k = 1, n
            t = WDOT(k-1,a(1,k),1,b(1),1)
            b(k) = (b(k) - t)/a(k,k)
         END DO
!        now solve trans(L)*x = y
         IF (n >= 2) THEN
         DO kb = 1, n-1
            k = n - kb
            b(k) = b(k) + WDOT(n-k,a(k+1,k),1,b(k+1),1)
            l = Ipvt(k)
            IF (l /= k) THEN
               t = b(l); b(l) = b(k); b(k) = t
            END IF
         END DO
         END IF
   
      END SELECT

      END SUBROUTINE WGESL
! End of BLAS_UTIL function
! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~



END MODULE gckpp_LinearAlgebra

